@@ -29,6 +29,7 @@
 #include <simgear/constants.h>
 #include <simgear/props/props.hxx>
 #include <simgear/props/props_io.hxx>
+#include <simgear/timing/sg_time.hxx>
 
 #include <Main/globals.hxx>
 #include <Main/fg_props.hxx>
@@ -41,6 +42,8 @@
 #include <Environment/environment_mgr.hxx>
 #include <Environment/environment.hxx>
 
+#include <Traffic/Schedule.hxx>
+
 #include "AIFlightPlan.hxx"
 #include "AIAircraft.hxx"
 
@@ -190,7 +193,7 @@ void FGAIFlightPlan::createWaypoints(FGAIAircraft *ac,
                                      const string& acType,
                                      const string& airline)
 {
-  time_t now = time(NULL) + fgGetLong("/sim/time/warp");
+  time_t now = globals->get_time_params()->get_cur_time();
   time_t timeDiff = now-start;
   leg = 1;
   
@@ -208,7 +211,12 @@ void FGAIFlightPlan::createWaypoints(FGAIAircraft *ac,
    if (timeDiff >= 2000)
    leg = 5;
    */
-  SG_LOG(SG_AI, SG_INFO, "Route from " << dep->getId() << " to " << arr->getId() << ". Set leg to : " << leg << " " << ac->getTrafficRef()->getCallSign());
+
+    if( sglog().would_log(SG_AI,SG_BULK) ) {
+        SG_LOG(SG_AI, SG_BULK, "Route from " << dep->getId() << " to " << arr->getId() <<
+               ". Set leg to : " << leg << " " << ac->getTrafficRef()->getCallSign());
+    }
+    
   wpt_iterator = waypoints.begin();
   bool dist = 0;
   isValid = create(ac, dep, arr, leg, alt, speed, lat, lon,
@@ -251,6 +259,11 @@ bool FGAIFlightPlan::parseProperties(const std::string& filename)
     wpt->setFinished   ((wpt->getName() == "END"));
     pushBackWaypoint( wpt );
   }
+  if( getLastWaypoint()->getName().compare("END") != 0  ) {
+	SG_LOG(SG_AI, SG_ALERT, "FGAIFlightPlan::Flightplan missing END node" );
+	return false;
+  }
+
   
   wpt_iterator = waypoints.begin();
   return true;
