@@ -40,6 +40,7 @@
 #include <Main/fg_props.hxx>
 #include <Main/locale.hxx>
 #include <Airports/runways.hxx>
+#include <Airports/groundnetwork.hxx>
 #include <Navaids/NavDataCache.hxx>
 
 #include "airport.hxx"
@@ -53,19 +54,19 @@ using std::random_shuffle;
 class ParkingAssignment::ParkingAssignmentPrivate
 {
 public:
-  ParkingAssignmentPrivate(FGParking* pk, FGAirport* apt) :
+  ParkingAssignmentPrivate(FGParking* pk, FGAirportDynamics* dyn) :
     refCount(0),
     parking(pk),
-    airport(apt)
+    dynamics(dyn)
   {
     assert(pk);
-    assert(apt);
+    assert(dyn);
     retain(); // initial count of 1
   }
   
   ~ParkingAssignmentPrivate()
   {
-    airport->getDynamics()->releaseParking(parking->guid());
+    dynamics->releaseParking(parking);
   }
   
   void release()
@@ -81,8 +82,8 @@ public:
   }
   
   unsigned int refCount;
-  SGSharedPtr<FGParking> parking;
-  SGSharedPtr<FGAirport> airport;
+  FGParkingRef parking;
+  FGAirportDynamicsRef dynamics;
 };
 
 ParkingAssignment::ParkingAssignment() :
@@ -97,11 +98,11 @@ ParkingAssignment::~ParkingAssignment()
   }
 }
   
-ParkingAssignment::ParkingAssignment(FGParking* pk, FGAirport* apt) :
+ParkingAssignment::ParkingAssignment(FGParking* pk, FGAirportDynamics* dyn) :
   _sharedData(NULL)
 {
   if (pk) {
-    _sharedData = new ParkingAssignmentPrivate(pk, apt);
+    _sharedData = new ParkingAssignmentPrivate(pk, dyn);
   }
 }
 
@@ -164,43 +165,49 @@ FGAirportDynamics::FGAirportDynamics(FGAirport * ap):
 // Destructor
 FGAirportDynamics::~FGAirportDynamics()
 {
+    SG_LOG(SG_AI, SG_INFO, "destroyed dynamics for:" << _ap->ident());
 }
 
 
 // Initialization required after XMLRead
 void FGAirportDynamics::init()
 {
-    groundNetwork.init(_ap);
-    groundNetwork.setTowerController(&towerController);
-    
+    groundController.setTowerController(&towerController);
+    groundController.init(this);
 }
 
 FGParking* FGAirportDynamics::innerGetAvailableParking(double radius, const string & flType,
                                            const string & airline,
                                            bool skipEmptyAirlineCode)
 {
-  flightgear::NavDataCache* cache = flightgear::NavDataCache::instance();
-  BOOST_FOREACH(PositionedID pk, cache->findAirportParking(_ap->guid(), flType, radius)) {
-    if (!isParkingAvailable(pk)) {
-      continue;
+    const FGParkingList& parkings(getGroundNetwork()->allParkings());
+    FGParkingList::const_iterator it;
+    for (it = parkings.begin(); it != parkings.end(); ++it) {
+        FGParkingRef parking = *it;
+        if (!isParkingAvailable(parking)) {
+          continue;
+        }
+
+        if (skipEmptyAirlineCode && parking->getCodes().empty()) {
+          continue;
+        }
+
+        if (!airline.empty() && !parking->getCodes().empty()) {
+          if (parking->getCodes().find(airline, 0) == string::npos) {
+            continue;
+          }
+        }
+
+        setParkingAvailable(parking, false);
+        return parking;
     }
-    
-    FGParking* parking = getParking(pk);
-    if (skipEmptyAirlineCode && parking->getCodes().empty()) {
-      continue;
-    }
-    
-    if (!airline.empty() && !parking->getCodes().empty()) {
-      if (parking->getCodes().find(airline, 0) == string::npos) {
-        continue;
-      }
-    }
-    
-    setParkingAvailable(pk, false);
-    return parking;
-  }
-  
-  return NULL;
+
+    return NULL;
+}
+
+bool FGAirportDynamics::hasParkings() const
+{
+    return !getGroundNetwork()->allParkings().empty();
 }
 
 ParkingAssignment FGAirportDynamics::getAvailableParking(double radius, const string & flType,
@@ -212,60 +219,53 @@ ParkingAssignment FGAirportDynamics::getAvailableParking(double radius, const st
   // most exact seach - airline codes must be present and match
   FGParking* result = innerGetAvailableParking(radius, flType, airline, true);
   if (result) {
-    return ParkingAssignment(result, _ap);
+    return ParkingAssignment(result, this);
   }
   
   // more tolerant - gates with empty airline codes are permitted
   result = innerGetAvailableParking(radius, flType, airline, false);
   if (result) {
-    return ParkingAssignment(result, _ap);
+    return ParkingAssignment(result, this);
   }
 
   // fallback - ignore the airline code entirely
   result = innerGetAvailableParking(radius, flType, string(), false);
-  return result ? ParkingAssignment(result, _ap) : ParkingAssignment();
-}
-
-FGParkingRef FGAirportDynamics::getParking(PositionedID id) const
-{
-  return FGPositioned::loadById<FGParking>(id);
-}
-
-string FGAirportDynamics::getParkingName(PositionedID id) const
-{
-  FGParking* p = getParking(id);
-  if (p) {
-    return p->getName();
-  }
-  
-  return string();
+  return result ? ParkingAssignment(result, this) : ParkingAssignment();
 }
 
 ParkingAssignment FGAirportDynamics::getParkingByName(const std::string& name) const
 {
-  PositionedID guid = flightgear::NavDataCache::instance()->airportItemWithIdent(parent()->guid(), FGPositioned::PARKING, name);
-  if (guid == 0) {
+    const FGParkingList& parkings(getGroundNetwork()->allParkings());
+    FGParkingList::const_iterator it;
+    for (it = parkings.begin(); it != parkings.end(); ++it) {
+        if ((*it)->name() == name) {
+            return ParkingAssignment(*it, const_cast<FGAirportDynamics*>(this));
+        }
+    }
+
     return ParkingAssignment();
-  }
-  
-  return ParkingAssignment(getParking(guid), _ap);
 }
 
-void FGAirportDynamics::setParkingAvailable(PositionedID guid, bool available)
+FGGroundNetwork *FGAirportDynamics::getGroundNetwork() const
+{
+    return _ap->groundNetwork();
+}
+
+void FGAirportDynamics::setParkingAvailable(FGParking* park, bool available)
 {
   if (available) {
-    releaseParking(guid);
+    releaseParking(park);
   } else {
-    occupiedParkings.insert(guid);
+    occupiedParkings.insert(park);
   }
 }
 
-bool FGAirportDynamics::isParkingAvailable(PositionedID parking) const
+bool FGAirportDynamics::isParkingAvailable(FGParking* parking) const
 {
   return (occupiedParkings.find(parking) == occupiedParkings.end());
 }
 
-void FGAirportDynamics::releaseParking(PositionedID id)
+void FGAirportDynamics::releaseParking(FGParking* id)
 {
   ParkingSet::iterator it = occupiedParkings.find(id);
   if (it == occupiedParkings.end()) {
@@ -275,11 +275,295 @@ void FGAirportDynamics::releaseParking(PositionedID id)
   occupiedParkings.erase(it);
 }
 
+class GetParkingsPredicate
+{
+    bool mustBeAvailable;
+    std::string type;
+    const FGAirportDynamics* dynamics;
+public:
+    GetParkingsPredicate(bool b, const std::string& ty, const FGAirportDynamics* dyn) :
+        mustBeAvailable(b),
+        type(ty),
+        dynamics(dyn)
+    {}
+
+    bool operator()(const FGParkingRef& park) const
+    {
+        if (!type.empty() && (park->getType() != type))
+            return true;
+
+        if (mustBeAvailable && !dynamics->isParkingAvailable(park)) {
+            return true;
+        }
+
+        return false;
+    }
+};
+
+FGParkingList FGAirportDynamics::getParkings(bool onlyAvailable, const std::string &type) const
+{
+    FGParkingList result(getGroundNetwork()->allParkings());
+
+    GetParkingsPredicate pred(onlyAvailable, type, this);
+    FGParkingList::iterator it = std::remove_if(result.begin(), result.end(), pred);
+    result.erase(it, result.end());
+    return result;
+}
+
 void FGAirportDynamics::setRwyUse(const FGRunwayPreference & ref)
 {
     rwyPrefs = ref;
 }
 
+bool areRunwaysParallel(const FGRunwayRef& a, const FGRunwayRef& b)
+{
+    double hdgDiff = (b->headingDeg() - a->headingDeg());
+    SG_NORMALIZE_RANGE(hdgDiff, -180.0, 180.0);
+    return (fabs(hdgDiff) < 5.0);
+}
+
+double runwayScore(const FGRunwayRef& rwy)
+{
+    return rwy->lengthM() + rwy->widthM();
+}
+
+double runwayWindScore(const FGRunwayRef& runway, double windHeading,
+                       double windSpeedKts)
+{
+    double hdgDiff = fabs(windHeading - runway->headingDeg()) * SG_DEGREES_TO_RADIANS;
+    SGMiscd::normalizeAngle(hdgDiff);
+
+    double crossWind = windSpeedKts * sin(hdgDiff);
+    double tailWind = -windSpeedKts * cos(hdgDiff);
+
+    return -(crossWind + tailWind);
+}
+
+typedef std::vector<FGRunwayRef> RunwayVec;
+
+
+class FallbackRunwayGroup
+{
+public:
+    FallbackRunwayGroup(const FGRunwayRef& rwy) :
+        _groupScore(0.0)
+    {
+        runways.push_back(rwy);
+        _leadRunwayScore = runwayScore(rwy);
+        _groupScore += _leadRunwayScore;
+    }
+
+    bool canAccept(const FGRunwayRef& rwy) const
+    {
+        if (!areRunwaysParallel(runways.front(), rwy)) {
+            return false;
+        }
+
+        return true;
+    }
+
+    void addRunway(const FGRunwayRef& rwy)
+    {
+        assert(areRunwaysParallel(runways.front(), rwy));
+        double score = runwayScore(rwy);
+        if (score < (0.5 * _leadRunwayScore)) {
+            // drop the runway completely. this is to drop short parallel
+            // runways from being active
+            return;
+        }
+
+        runways.push_back(rwy);
+        _groupScore += score;
+    }
+
+    void adjustScoreForWind(double windHeading, double windSpeedKts)
+    {
+        _basicScore = _groupScore;
+        RunwayVec::iterator it;
+        for (it = runways.begin(); it != runways.end(); ++it) {
+            _groupScore += runwayWindScore(*it, windHeading, windSpeedKts);
+        }
+    }
+
+    double groupScore() const
+    {
+        return _groupScore;
+    }
+
+    void getRunways(FGRunwayList& arrivals, FGRunwayList& departures)
+    {
+    // make the common cases very obvious
+        if (runways.size() == 1) {
+            arrivals.push_back(runways.front());
+            departures.push_back(runways.front());
+            return;
+        }
+
+
+    // becuase runways were sorted by score when building, they were added
+    // by score also, so we can use a simple algorithim to assign
+        for (unsigned int r=0; r < runways.size(); ++r) {
+            if ((r % 2) == 0) {
+                arrivals.push_back(runways[r]);
+            } else {
+                departures.push_back(runways[r]);
+            }
+        }
+    }
+
+    std::string dump()
+    {
+        ostringstream os;
+        os << runways.front()->ident();
+        for (unsigned int r=1; r <runways.size(); ++r) {
+            os << ", " << runways[r]->ident();
+        }
+
+        os << " (score=" << _basicScore << ", wind score=" << _groupScore << ")";
+        return os.str();
+    }
+
+private:
+    RunwayVec runways;
+    double _groupScore,
+        _leadRunwayScore;
+    double _basicScore;
+
+};
+
+class WindExclusionCheck
+{
+public:
+    WindExclusionCheck(double windHeading, double windSpeedKts) :
+        _windSpeedKts(windSpeedKts),
+        _windHeading(windHeading)
+    {}
+
+    bool operator()(const FGRunwayRef& rwy) const
+    {
+        return (runwayWindScore(rwy, _windHeading, _windSpeedKts) > 30);
+    }
+private:
+    double _windSpeedKts,
+        _windHeading;
+};
+
+class SortByScore
+{
+public:
+    bool operator()(const FGRunwayRef& a, const FGRunwayRef& b) const
+    {
+        return runwayScore(a) > runwayScore(b);
+    }
+};
+
+class GroupSortByScore
+{
+public:
+    bool operator()(const FallbackRunwayGroup& a, const FallbackRunwayGroup& b) const
+    {
+        return a.groupScore() > b.groupScore();
+    }
+};
+
+string FGAirportDynamics::fallbackGetActiveRunway(int action, double heading)
+{
+    bool updateNeeded = false;
+    if (_lastFallbackUpdate == SGTimeStamp()) {
+        updateNeeded = true;
+    } else {
+        updateNeeded = (_lastFallbackUpdate.elapsedMSec() > (1000 * 60 * 15));
+    }
+
+    if (updateNeeded) {
+        double windSpeed   = fgGetInt("/environment/metar/base-wind-speed-kt");
+        double windHeading = fgGetInt("/environment/metar/base-wind-dir-deg");
+
+        // discount runways based on cross / tail-wind
+        WindExclusionCheck windCheck(windHeading, windSpeed);
+        RunwayVec runways(parent()->getRunways());
+        RunwayVec::iterator it = std::remove_if(runways.begin(), runways.end(),
+                                                windCheck);
+        runways.erase(it, runways.end());
+
+        // sort highest scored to lowest scored
+        std::sort(runways.begin(), runways.end(), SortByScore());
+
+        std::vector<FallbackRunwayGroup> groups;
+        std::vector<FallbackRunwayGroup>::iterator git;
+
+        for (it = runways.begin(); it != runways.end(); ++it) {
+            bool existingGroupDidAccept = false;
+            for (git = groups.begin(); git != groups.end(); ++git) {
+                if (git->canAccept(*it)) {
+                    existingGroupDidAccept = true;
+                    git->addRunway(*it);
+                    break;
+                }
+            } // of existing groups iteration
+
+            if (!existingGroupDidAccept) {
+                // create a new group
+                groups.push_back(FallbackRunwayGroup(*it));
+            }
+        } // of group building phase
+
+
+        // select highest scored group based on cross/tail wind
+        for (git = groups.begin(); git != groups.end(); ++git) {
+            git->adjustScoreForWind(windHeading, windSpeed);
+        }
+
+        std::sort(groups.begin(), groups.end(), GroupSortByScore());
+
+        {
+            ostringstream os;
+            os << parent()->ident() << " groups:";
+
+            for (git = groups.begin(); git != groups.end(); ++git) {
+                os << "\n\t" << git->dump();
+            }
+
+            std::string s = os.str();
+            SG_LOG(SG_AI, SG_INFO, s);
+        }
+
+        // assign takeoff and landing runways
+        FallbackRunwayGroup bestGroup = groups.front();
+        
+        _lastFallbackUpdate.stamp();
+        _fallbackRunwayCounter = 0;
+        _fallbackDepartureRunways.clear();
+        _fallbackArrivalRunways.clear();
+        bestGroup.getRunways(_fallbackArrivalRunways, _fallbackDepartureRunways);
+
+        ostringstream os;
+        os << "\tArrival:" << _fallbackArrivalRunways.front()->ident();
+        for (unsigned int r=1; r <_fallbackArrivalRunways.size(); ++r) {
+            os << ", " << _fallbackArrivalRunways[r]->ident();
+        }
+        os << "\n\tDeparture:" << _fallbackDepartureRunways.front()->ident();
+        for (unsigned int r=1; r <_fallbackDepartureRunways.size(); ++r) {
+            os << ", " << _fallbackDepartureRunways[r]->ident();
+        }
+
+        std::string s = os.str();
+        SG_LOG(SG_AI, SG_INFO, parent()->ident() << " fallback runways assignments for "
+               << static_cast<int>(windHeading) << "@" << static_cast<int>(windSpeed) << "\n" << s);
+    }
+
+    _fallbackRunwayCounter++;
+    FGRunwayRef r;
+    // ensure we cycle through possible runways where they exist
+    if (action == 1) {
+        r = _fallbackDepartureRunways[_fallbackRunwayCounter % _fallbackDepartureRunways.size()];
+    } else {
+        r = _fallbackArrivalRunways[_fallbackRunwayCounter % _fallbackArrivalRunways.size()];
+    }
+
+    return r->ident();
+}
+
 bool FGAirportDynamics::innerGetActiveRunway(const string & trafficType,
                                              int action, string & runway,
                                              double heading)
@@ -292,13 +576,14 @@ bool FGAirportDynamics::innerGetActiveRunway(const string & trafficType,
     string type;
 
     if (!rwyPrefs.available()) {
-        return false;
+        runway = fallbackGetActiveRunway(action, heading);
+        return true;
     }
 
     RunwayGroup *currRunwayGroup = 0;
     int nrActiveRunways = 0;
     time_t dayStart = fgGetLong("/sim/time/utc/day-seconds");
-    if ((abs((long) (dayStart - lastUpdate)) > 600)
+    if ((std::abs((long) (dayStart - lastUpdate)) > 600)
         || trafficType != prevTrafficType) {
         landing.clear();
         takeoff.clear();
@@ -484,6 +769,9 @@ int FGAirportDynamics::getGroundFrequency(unsigned leg)
         SG_LOG(SG_ATC, SG_ALERT,
                "Leg value is smaller than one at " << SG_ORIGIN);
     }
+
+    const intVec& freqGround(getGroundNetwork()->getGroundFrequencies());
+
     if (freqGround.size() == 0) {
         return 0;
     }
@@ -507,6 +795,9 @@ int FGAirportDynamics::getTowerFrequency(unsigned nr)
         SG_LOG(SG_ATC, SG_ALERT,
                "Leg value is smaller than two at " << SG_ORIGIN);
     }
+
+    const intVec& freqTower(getGroundNetwork()->getTowerFrequencies());
+
     if (freqTower.size() == 0) {
         return 0;
     }
