@@ -229,6 +229,8 @@ FGJSBsim::FGJSBsim( double dt )
     fgGetNode("/fdm/jsbsim/sim-time-sec", true)->alias( node );
 // end of sim-time-sec deprecation patch
 
+    terrain = fgGetNode("/sim/fdm/surface", true);
+
     fdmex->Setdt( dt );
 
     result = fdmex->LoadModel( aircraft_path.str(),
@@ -523,7 +525,12 @@ void FGJSBsim::update( double dt )
     trimmed->setBoolValue(false);
 
     for ( int i=0; i < multiloop; i++ ) {
-      fdmex->Run();
+      if (!fdmex->Run()) {
+        // The property fdm/jsbsim/simulation/terminate has been set to true
+        // by the user. The sim is considered crashed.
+        crashed = true;
+        break;
+      }
       update_external_forces(fdmex->GetSimTime() + i * fdmex->GetDeltaT());
     }
 
@@ -956,6 +963,8 @@ bool FGJSBsim::copy_from_JSBsim()
         node->setDoubleValue("density-ppg" , fuelDensity);
         node->setDoubleValue("level-lbs", contents);
         if (temp != -9999.0) node->setDoubleValue("temperature_degC", temp);
+
+        node->setDoubleValue("arm-in", tank->GetXYZ(FGJSBBase::eX ) );
       }
     }
 
@@ -1345,8 +1354,6 @@ FGJSBsim::get_agl_ft(double t, const double pt[3], double alt_off,
   SGQuatd hlToEc = SGQuatd::fromLonLat(geodPt);
   *agl = dot(hlToEc.rotate(SGVec3d(0, 0, 1)), SGVec3d(contact) - SGVec3d(pt));
 
-  static SGPropertyNode_ptr terrain = fgGetNode("/sim/fdm/surface", true);
-
 #ifdef JSBSIM_USE_GROUNDREACTIONS
   bool terrain_active = (terrain->getIntValue("override-level", -1) > 0) ? false : true;
   terrain->setBoolValue("active", terrain_active);
