@@ -44,13 +44,12 @@ INCLUDES
 #include "math/FGColumnVector3.h"
 #include "math/LagrangeMultiplier.h"
 #include "math/FGMatrix33.h"
-#include "math/FGQuaternion.h"
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 DEFINITIONS
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-#define ID_ACCELERATIONS "$Id: FGAccelerations.h,v 1.15 2013/11/29 18:56:30 jberndt Exp $"
+#define ID_ACCELERATIONS "$Id: FGAccelerations.h,v 1.19 2016/04/16 12:24:39 bcoconni Exp $"
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 FORWARD DECLARATIONS
@@ -97,7 +96,7 @@ CLASS DOCUMENTATION
          NASA SP-8024, May 1969
 
     @author Jon S. Berndt, Mathias Froehlich, Bertrand Coconnier
-    @version $Id: FGAccelerations.h,v 1.15 2013/11/29 18:56:30 jberndt Exp $
+    @version $Id: FGAccelerations.h,v 1.19 2016/04/16 12:24:39 bcoconni Exp $
   */
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -135,17 +134,6 @@ public:
       @return false if no error */
   bool Run(bool Holding);
 
-  /** Retrieves the time derivative of the body orientation quaternion.
-      Retrieves the time derivative of the body orientation quaternion based on
-      the rate of change of the orientation between the body and the ECI frame.
-      The quaternion returned is represented by an FGQuaternion reference. The
-      quaternion is 1-based, so that the first element can be retrieved using
-      the "()" operator.
-      units rad/sec^2
-      @return The time derivative of the body orientation quaternion.
-  */
-  const FGQuaternion& GetQuaterniondot(void) const {return vQtrndot;}
-
   /** Retrieves the body axis acceleration.
       Retrieves the computed body axis accelerations based on the
       applied forces and accounting for a rotating body frame.
@@ -327,12 +315,8 @@ public:
   /** Sets the property forces/hold-down. This allows to do hard 'hold-down'
       such as for rockets on a launch pad with engines ignited.
       @param hd enables the 'hold-down' function if non-zero
-   */
-  void SetHoldDown(int hd) {HoldDown = hd;}
-  /** Gets the value of the property forces/hold-down.
-      @result zero if the 'hold-down' function is disabled, non-zero otherwise.
-   */
-  int GetHoldDown(void) const {return HoldDown;}
+  */
+  void SetHoldDown(bool hd);
 
   struct Inputs {
     /// The body inertia matrix expressed in the body frame
@@ -347,8 +331,6 @@ public:
     FGMatrix33 Tec2b;
     /// Transformation matrix from the ECEF to the ECI frame
     FGMatrix33 Tec2i;
-    /// Orientation quaternion of the body with respect to the ECI frame
-    FGQuaternion qAttitudeECI;
     /// Total moments applied to the body except friction and gravity (expressed in the body frame)
     FGColumnVector3 Moment;
     /// Moments generated by the ground normal reactions expressed in the body frame. Does not account for friction.
@@ -387,7 +369,6 @@ private:
 
   FGColumnVector3 vPQRdot, vPQRidot;
   FGColumnVector3 vUVWdot, vUVWidot;
-  FGQuaternion vQtrndot;
   FGColumnVector3 vBodyAccel;
   FGColumnVector3 vGravAccel;
   FGColumnVector3 vFrictionForces;
@@ -395,10 +376,8 @@ private:
 
   int gravType;
   bool gravTorque;
-  int HoldDown;
 
   void CalculatePQRdot(void);
-  void CalculateQuatdot(void);
   void CalculateUVWdot(void);
 
   void ResolveFrictionForces(double dt);
