@@ -51,7 +51,7 @@ using namespace std;
 
 namespace JSBSim {
 
-IDENT(IdSrc,"$Id: FGAuxiliary.cpp,v 1.67 2014/05/17 15:28:51 jberndt Exp $");
+IDENT(IdSrc,"$Id: FGAuxiliary.cpp,v 1.71 2016/01/10 12:12:59 bcoconni Exp $");
 IDENT(IdHdr,ID_AUXILIARY);
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -68,23 +68,24 @@ FGAuxiliary::FGAuxiliary(FGFDMExec* fdmex) : FGModel(fdmex)
 
   vcas = veas = 0.0;
   qbar = qbarUW = qbarUV = 0.0;
-  Mach = MachU = 0.0;
+  Mach = MachU = MachPitot = 0.0;
   alpha = beta = 0.0;
   adot = bdot = 0.0;
-  gamma = Vt = Vground = 0.0;
+  gamma = Vt = Vground = Vpitot = 0.0;
   psigt = 0.0;
   day_of_year = 1;
   seconds_in_day = 0.0;
   hoverbmac = hoverbcg = 0.0;
   Re = 0.0;
   Nz = Ny = 0.0;
-  lon_relative_position = lat_relative_position = relative_position = 0.0;
 
   vPilotAccel.InitMatrix();
   vPilotAccelN.InitMatrix();
   vAeroUVW.InitMatrix();
   vAeroPQR.InitMatrix();
   vMachUVW.InitMatrix();
+  vWindUVW.InitMatrix();
+  vPitotUVW.InitMatrix();
   vEuler.InitMatrix();
   vEulerRates.InitMatrix();
 
@@ -105,17 +106,16 @@ bool FGAuxiliary::InitModel(void)
 
   vcas = veas = 0.0;
   qbar = qbarUW = qbarUV = 0.0;
-  Mach = MachU = 0.0;
+  Mach = MachU = MachPitot = 0.0;
   alpha = beta = 0.0;
   adot = bdot = 0.0;
-  gamma = Vt = Vground = 0.0;
+  gamma = Vt = Vground = Vpitot = 0.0;
   psigt = 0.0;
   day_of_year = 1;
   seconds_in_day = 0.0;
   hoverbmac = hoverbcg = 0.0;
   Re = 0.0;
   Nz = Ny = 0.0;
-  lon_relative_position = lat_relative_position = relative_position = 0.0;
 
   vPilotAccel.InitMatrix();
   vPilotAccelN.InitMatrix();
@@ -139,8 +139,6 @@ FGAuxiliary::~FGAuxiliary()
 
 bool FGAuxiliary::Run(bool Holding)
 {
-  double A,B,D;
-
   if (FGModel::Run(Holding)) return true; // return true if error returned from base class
   if (Holding) return false;
 
@@ -196,7 +194,6 @@ bool FGAuxiliary::Run(bool Holding)
   MachU = vMachUVW(eU) = vAeroUVW(eU) / in.SoundSpeed;
   vMachUVW(eV) = vAeroUVW(eV) / in.SoundSpeed;
   vMachUVW(eW) = vAeroUVW(eW) / in.SoundSpeed;
-  double MachU2 = MachU * MachU;
 
   // Position
 
@@ -209,18 +206,17 @@ bool FGAuxiliary::Run(bool Holding)
   tat = in.Temperature*(1 + 0.2*Mach*Mach); // Total Temperature, isentropic flow
   tatc = RankineToCelsius(tat);
 
-  if (MachU < 1) {   // Calculate total pressure assuming isentropic flow
-    pt = in.Pressure*pow((1 + 0.2*MachU2),3.5);
-  } else {
-    // Use Rayleigh pitot tube formula for normal shock in front of pitot tube
-    B = 5.76 * MachU2 / (5.6*MachU2 - 0.8);
-    D = (2.8 * MachU2 - 0.4) * 0.4167;
-    pt = in.Pressure*pow(B,3.5)*D;
-  }
+  // Pitot
 
-  A = pow(((pt-in.Pressure)/in.PressureSL + 1),0.28571);
-  if (abs(MachU) > 0.0) {
-    vcas = sqrt(7 * in.PressureSL / in.DensitySL * (A-1));
+  vWindUVW(eU) = Vt;
+  vPitotUVW = mTw2p * vWindUVW;
+  Vpitot = vPitotUVW(eU);
+  if (Vpitot < 0.0) Vpitot = 0.0;
+  MachPitot = Vpitot / in.SoundSpeed;
+  pt = PitotTotalPressure(MachPitot, in.Pressure);
+
+  if (abs(MachPitot) > 0.0) {
+    vcas = VcalibratedFromMach(MachPitot, in.Pressure, in.PressureSL, in.DensitySL);
     veas = sqrt(2 * qbar / in.DensitySL);
     vtrue = 1116.43559 * Mach * sqrt(in.Temperature / 518.67);
   } else {
@@ -232,8 +228,8 @@ bool FGAuxiliary::Run(bool Holding)
   // Nz is Acceleration in "g's", along normal axis (-Z body axis)
   Nz = -vNcg(eZ);
   Ny =  vNcg(eY);
-  vPilotAccel = in.vBodyAccel + in.vPQRdot * in.ToEyePt;
-  vPilotAccel += in.vPQR * (in.vPQR * in.ToEyePt);
+  vPilotAccel = in.vBodyAccel + in.vPQRidot * in.ToEyePt;
+  vPilotAccel += in.vPQRi * (in.vPQRi * in.ToEyePt);
 
   vNwcg = mTb2w * vNcg;
   vNwcg(eZ) = 1.0 - vNwcg(eZ);
@@ -249,9 +245,6 @@ bool FGAuxiliary::Run(bool Holding)
   FGColumnVector3 vMac = in.Tb2l * in.RPBody;
   hoverbmac = (in.DistanceAGL + vMac(3)) / in.Wingspan;
 
-  // When all models are executed calculate the distance from the initial point.
-  CalculateRelativePosition();
-
   return false;
 }
 
@@ -291,6 +284,23 @@ void FGAuxiliary::UpdateWindMatrices(void)
   mTw2b(3,3) =  ca;
 
   mTb2w = mTw2b.Transposed();
+
+  // The pitot frame is the same as the body frame except rotated about the
+  // Y axis by the pitot attachment angle.
+
+  ca = cos(alpha + in.PitotAngle);
+  sa = sin(alpha + in.PitotAngle);
+
+  mTw2p(1,1) =  ca*cb;
+  mTw2p(1,2) = -ca*sb;
+  mTw2p(1,3) = -sa;
+  mTw2p(2,1) =  sb;
+  mTw2p(2,2) =  cb;
+  mTw2p(2,3) =  0.0;
+  mTw2p(3,1) =  sa*cb;
+  mTw2p(3,2) = -sa*sb;
+  mTw2p(3,3) =  ca;
+
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -317,13 +327,6 @@ double FGAuxiliary::GetCrossWind(void) const
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
-double FGAuxiliary::GethVRP(void) const
-{
-  return vLocationVRP.GetRadius() - in.ReferenceRadius;
-}
-
-//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-
 double FGAuxiliary::GetNlf(void) const
 {
   if (in.Mass != 0)
@@ -334,13 +337,36 @@ double FGAuxiliary::GetNlf(void) const
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
-void FGAuxiliary::CalculateRelativePosition(void)  //ToDo: This belongs elsewhere - perhaps in FGPropagate or Exec
+double FGAuxiliary::GetLongitudeRelativePosition(void) const
 {
-  const double earth_radius_mt = in.ReferenceRadius*fttom;
-  lat_relative_position=(in.vLocation.GetLatitude()  - FDMExec->GetIC()->GetLatitudeDegIC() *degtorad)*earth_radius_mt;
-  lon_relative_position=(in.vLocation.GetLongitude() - FDMExec->GetIC()->GetLongitudeDegIC()*degtorad)*earth_radius_mt;
-  relative_position = sqrt(lat_relative_position*lat_relative_position + lon_relative_position*lon_relative_position);
-};
+  FGLocation source(FDMExec->GetIC()->GetLongitudeRadIC(),
+                    FDMExec->GetIC()->GetLatitudeRadIC(),
+                    in.vLocation.GetSeaLevelRadius());
+  return source.GetDistanceTo(in.vLocation.GetLongitude(),
+                              FDMExec->GetIC()->GetLatitudeRadIC()) * fttom;
+}
+
+//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
+double FGAuxiliary::GetLatitudeRelativePosition(void) const
+{
+  FGLocation source(FDMExec->GetIC()->GetLongitudeRadIC(),
+                    FDMExec->GetIC()->GetLatitudeRadIC(),
+                    in.vLocation.GetSeaLevelRadius());
+  return source.GetDistanceTo(FDMExec->GetIC()->GetLongitudeRadIC(),
+                              in.vLocation.GetLatitude()) * fttom;
+}
+
+//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
+double FGAuxiliary::GetDistanceRelativePosition(void) const
+{
+  FGLocation source(FDMExec->GetIC()->GetLongitudeRadIC(),
+                    FDMExec->GetIC()->GetLatitudeRadIC(),
+                    in.vLocation.GetSeaLevelRadius());
+  return source.GetDistanceTo(in.vLocation.GetLongitude(),
+                              in.vLocation.GetLatitude()) * fttom;
+}
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
