@@ -40,9 +40,8 @@ INCLUDES
 
 #include <iosfwd>
 #include <vector>
-
 #include <string>
-#include "models/flight_control/FGFCSComponent.h"
+
 #include "models/FGModel.h"
 #include "models/FGLGear.h"
 
@@ -50,7 +49,7 @@ INCLUDES
 DEFINITIONS
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-#define ID_FCS "$Id: FGFCS.h,v 1.46 2014/06/09 11:52:07 bcoconni Exp $"
+#define ID_FCS "$Id: FGFCS.h,v 1.50 2016/02/27 16:54:15 bcoconni Exp $"
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 FORWARD DECLARATIONS
@@ -168,7 +167,7 @@ CLASS DOCUMENTATION
     @property gear/tailhook-pos-norm
 
     @author Jon S. Berndt
-    @version $Revision: 1.46 $
+    @version $Revision: 1.50 $
     @see FGActuator
     @see FGDeadBand
     @see FGFCSFunction
@@ -551,23 +550,19 @@ public:
   /** Loads the Flight Control System.
       Load() is called from FGFDMExec.
       @param el pointer to the Element instance
-      @param systype type of system (FCS, Autopilot, System) 
       @return true if succesful */
-  bool Load(Element* el, SystemType systype);
+  bool Load(Element* el);
 
   std::string FindFullPathName(const std::string& system_filename) const;
 
   void AddThrottle(void);
   void AddGear(unsigned int NumGear);
-  double GetDt(void);
+  double GetDt(void) const;
 
   FGPropertyManager* GetPropertyManager(void) { return PropertyManager; }
 
   bool GetTrimStatus(void) const { return FDMExec->GetTrimStatus(); }
-
-  struct Inputs {
-    unsigned int NumGear;
-  } in;
+  double GetChannelDeltaT(void) const { return GetDt() * ChannelRate; }
 
 private:
   double DaCmd, DeCmd, DrCmd, DsCmd, DfCmd, DsbCmd, DspCmd;
@@ -583,11 +578,12 @@ private:
   std::vector <bool> PropFeatherCmd;
   std::vector <bool> PropFeather;
   std::vector <double> SteerPosDeg;
-  double LeftBrake, RightBrake, CenterBrake; // Brake settings
+  //double LeftBrake, RightBrake, CenterBrake; // Brake settings
   std::vector <double> BrakePos; // left, center, right - defined by FGLGear:: enum
   double GearCmd,GearPos;
   double TailhookPos, WingFoldPos;
   SystemType systype;
+  int ChannelRate;
 
   typedef std::vector <FGFCSChannel*> Channels;
   Channels SystemChannels;
