@@ -39,7 +39,7 @@
 #include <Main/globals.hxx>
 #include <Main/fg_props.hxx>
 #include <Viewer/viewmgr.hxx>
-#include <Viewer/viewer.hxx>
+#include <Viewer/view.hxx>
 #include <GUI/FGFontCache.hxx>
 #include <GUI/gui.h> // for guiErrorMessage
 
@@ -130,11 +130,10 @@ HUD::~HUD()
 
 void HUD::init()
 {
-    const char* fontName = 0;
-    _font_cache = globals->get_fontcache();
+    std::string fontName;
     if (!_font) {
         fontName = fgGetString("/sim/hud/font/name", "Helvetica.txf");
-        _font = _font_cache->getTexFont(fontName);
+        _font = FGFontCache::instance()->getTexFont(fontName);
     }
     if (!_font)
         throw sg_io_exception("/sim/hud/font/name is not a texture font",
@@ -214,7 +213,7 @@ void HUD::draw(osg::State&)
 void HUD::draw3D()
 {
     using namespace osg;
-    FGViewer* view = globals->get_current_view();
+    flightgear::View* view = globals->get_current_view();
 
     // Standard fgfs projection, with essentially meaningless clip
     // planes (we'll map the whole HUD plane to z=-1)
