@@ -91,7 +91,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <assert.h>
-#include <math.h>
+#include <cmath>
 
 #include <string>
 #include <sstream>
@@ -106,11 +106,7 @@
 
 using std::string;
 
-#if defined( HAVE_VERSION_H ) && HAVE_VERSION_H
-#  include <Include/version.h>
-#else
-#  include <Include/no_version.h>
-#endif
+#include <Include/version.h>
 
 ///////////////////////////////////////////////////////////////////////////////
 // debug switches /////////////////////////////////////////////////////////////
