@@ -50,6 +50,7 @@
 #include <stdio.h>
 #include <cstring>
 #include <iostream>
+#include <locale.h>
 
 #include <simgear/compiler.h>
 #include <simgear/structure/exception.hxx>
@@ -67,10 +68,6 @@
 
 #include "fg_os.hxx"
 
-#if defined(SG_MAC)
-    #include <GUI/CocoaHelpers.h> // for transformToForegroundApp
-#endif
-
 #if defined(HAVE_CRASHRPT)
 	#include <CrashRpt.h>
 
@@ -208,24 +205,37 @@ int main ( int argc, char **argv )
 		char buf[16];
 		::snprintf(buf, 16, "%d", HUDSON_BUILD_NUMBER);
 		crAddProperty("hudson-build-number", buf);
+        crAddProperty("git-revision", REVISION);
+#if defined(FG_NIGHTLY)
+        crAddProperty("nightly-build", "true");
+#endif
 	}
 #endif
 
     initFPE(flightgear::Options::checkForArg(argc, argv, "enable-fpe"));
 
+    // pick up all user locale settings, but force C locale for numerical/sorting
+    // conversions because we have lots of code which assumes standard
+    // formatting
+    setlocale(LC_ALL, "");
+    setlocale(LC_NUMERIC, "C");
+    setlocale(LC_COLLATE, "C");
+
     bool fgviewer = flightgear::Options::checkForArg(argc, argv, "fgviewer");
     try {
         // http://code.google.com/p/flightgear-bugs/issues/detail?id=1231
         // ensure sglog is inited before atexit() is registered, so logging
         // is possible inside fgExitCleanup
         sglog();
-        
+
+
+#if (OPENSCENEGRAPH_MAJOR_VERSION == 3) && (OPENSCENEGRAPH_MINOR_VERSION < 5)
         // similar to above, ensure some static maps inside OSG exist before
         // we register our at-exit handler, otherwise the statics are gone
         // when fg_terminate runs, which causes crashes.
         osg::Texture::getTextureObjectManager(0);
         osg::GLBufferObjectManager::getGLBufferObjectManager(0);
-        
+#endif
         std::set_terminate(fg_terminate);
         atexit(fgExitCleanup);
         if (fgviewer)
