@@ -68,6 +68,7 @@
 #include <simgear/scene/material/Effect.hxx>
 #include <simgear/scene/model/particles.hxx>
 #include <simgear/scene/tsync/terrasync.hxx>
+#include <simgear/scene/tgdb/userdata.hxx>
 
 #include <simgear/package/Root.hxx>
 #include <simgear/package/Package.hxx>
@@ -80,6 +81,7 @@
 #include <Airports/runways.hxx>
 #include <Airports/airport.hxx>
 #include <Airports/dynamics.hxx>
+#include <Airports/airportdynamicsmanager.hxx>
 
 #include <AIModel/AIManager.hxx>
 
@@ -102,6 +104,7 @@
 #include <Model/modelmgr.hxx>
 #include <AIModel/submodel.hxx>
 #include <AIModel/AIManager.hxx>
+#include <AIModel/performancedb.hxx>
 #include <Navaids/navdb.hxx>
 #include <Navaids/navlist.hxx>
 #include <Scenery/scenery.hxx>
@@ -129,7 +132,7 @@
 #include <Network/fgcom.hxx>
 #include <Network/http/httpd.hxx>
 #include <Include/version.h>
-
+#include <Viewer/splash.hxx>
 #include <Viewer/CameraGroup.hxx>
 
 #include "fg_init.hxx"
@@ -160,14 +163,14 @@ using namespace simgear::pkg;
 extern osg::ref_ptr<osgViewer::Viewer> viewer;
 
 // Return the current base package version
-string fgBasePackageVersion() {
-    SGPath base_path( globals->get_fg_root() );
-    base_path.append("version");
-    if (!base_path.exists()) {
+string fgBasePackageVersion(const SGPath& base_path) {
+    SGPath p(base_path);
+    p.append("version");
+    if (!p.exists()) {
         return string();
     }
     
-    sg_gzifstream in( base_path.str() );
+    sg_gzifstream in( p.str() );
     if (!in.is_open()) {
         return string();
     }
@@ -390,7 +393,7 @@ bool fgInitHome()
 // write our PID, and check writeability
     SGPath pidPath(dataPath, "fgfs.pid");
     if (pidPath.exists()) {
-        SG_LOG(SG_GENERAL, SG_INFO, "flightgear instance already running, switching to FG_HOME read-only.");
+        SG_LOG(SG_GENERAL, SG_ALERT, "flightgear instance already running, switching to FG_HOME read-only.");
         // set a marker property so terrasync/navcache don't try to write
         // from secondary instances
         fgSetBool("/sim/fghome-readonly", true);
@@ -446,8 +449,9 @@ int fgInitConfig ( int argc, char **argv, bool reinit )
       exportDir.create(0755);
     }
     
-    // Set /sim/fg-home and don't allow malign code to override it until
-    // Nasal security is set up.  Use FG_HOME if necessary.
+    // Set /sim/fg-home.  Use FG_HOME if necessary.
+    // deliberately not a tied property, for fgValidatePath security
+    // write-protect to avoid accidents
     SGPropertyNode *home = fgGetNode("/sim", true);
     home->removeChild("fg-home", 0);
     home = home->getChild("fg-home", 0, true);
@@ -486,6 +490,8 @@ int fgInitConfig ( int argc, char **argv, bool reinit )
 
 static void initAircraftDirsNasalSecurity()
 {
+    // deliberately not a tied property, for fgValidatePath security
+    // write-protect to avoid accidents
     SGPropertyNode* sim = fgGetNode("/sim", true);
     sim->removeChildren("fg-aircraft");
 
@@ -503,17 +509,8 @@ static void initAircraftDirsNasalSecurity()
 
 void fgInitAircraftPaths(bool reinit)
 {
-  if (!reinit) {
-      // there is some debate if we should be using FG_HOME here (hidden
-      // location) vs a user-visible location inside Documents (especially on
-      // Windows and Mac). Really this location should be managed by FG, not
-      // the user, but it can potentially grow large.
-      SGPath packageAircraftDir = globals->get_fg_home();
-      packageAircraftDir.append("Aircraft");
-
-      SGSharedPtr<Root> pkgRoot(new Root(packageAircraftDir, FLIGHTGEAR_VERSION));
-      // set the http client later (too early in startup right now)
-      globals->setPackageRoot(pkgRoot);
+  if (!globals->packageRoot()) {
+      fgInitPackageRoot();
   }
 
   SGSharedPtr<Root> pkgRoot(globals->packageRoot());
@@ -554,9 +551,9 @@ int fgInitAircraft(bool reinit)
             size_t lastDot = aircraftId.rfind('.');
             if (lastDot != std::string::npos) {
                 aircraftId = aircraftId.substr(lastDot + 1);
-                aircraftProp->setStringValue(aircraftId);
-
             }
+            aircraftProp->setStringValue(aircraftId);
+
             // run the traditional-code path below
         } else {
 #if 0
@@ -567,6 +564,7 @@ int fgInitAircraft(bool reinit)
 
             return flightgear::FG_OPTIONS_ERROR;
 #endif
+            // fall back the default aircraft instead
         }
     }
 
@@ -590,22 +588,36 @@ int fgInitAircraft(bool reinit)
 bool
 fgInitNav ()
 {
-  flightgear::NavDataCache* cache = flightgear::NavDataCache::instance();
-  static bool doingRebuild = false;
-  if (doingRebuild || cache->isRebuildRequired()) {
-    doingRebuild = true;
-    bool finished = cache->rebuild();
-    if (!finished) {
-      // sleep to give the rebuild thread more time
-      SGTimeStamp::sleepForMSec(50);
-      return false;
-    }
-  }
+    flightgear::NavDataCache* cache = flightgear::NavDataCache::instance();
+    static bool doingRebuild = false;
 
-    // depend on when the NavCache was initialised, scenery paths may not
-    // have been setup. This is a safe place to consistently check the value,
-    // and drop the ground-nets if something has changed
-    cache->dropGroundnetsIfRequired();
+    if (!cache) {
+        cache = flightgear::NavDataCache::createInstance();
+        doingRebuild = cache->isRebuildRequired();
+    }
+
+    static const char* splashIdentsByRebuildPhase[] = {
+        "loading-nav-dat",
+        "navdata-airports",
+        "navdata-navaids",
+        "navdata-fixes",
+        "navdata-pois"
+    };
+
+    if (doingRebuild) {
+        flightgear::NavDataCache::RebuildPhase phase;
+        phase = cache->rebuild();
+        if (phase != flightgear::NavDataCache::REBUILD_DONE) {
+            // update the splash text based on percentage, phase
+
+            fgSplashProgress(splashIdentsByRebuildPhase[phase],
+                             cache->rebuildPhaseCompletionPercentage());
+
+            // sleep to give the rebuild thread more time
+            SGTimeStamp::sleepForMSec(50);
+            return false;
+        }
+    }
 
   FGTACANList *channellist = new FGTACANList;
   globals->set_channellist( channellist );
@@ -647,6 +659,9 @@ bool fgInitGeneral() {
 
     fgSetBool("/sim/startup/stdout-to-terminal", isatty(1) != 0 );
     fgSetBool("/sim/startup/stderr-to-terminal", isatty(2) != 0 );
+
+    sgUserDataInit( globals->get_props() );
+
     return true;
 }
 
@@ -658,8 +673,9 @@ void fgOutputSettings()
     
     SG_LOG( SG_GENERAL, SG_INFO, "aircraft-dir = " << '"' << fgGetString("/sim/aircraft-dir") << '"' );
     SG_LOG( SG_GENERAL, SG_INFO, "fghome-dir = " << '"' << globals->get_fg_home() << '"');
-    SG_LOG( SG_GENERAL, SG_INFO, "aircraft-dir = " << '"' << fgGetString("/sim/aircraft-dir") << '"');
-    
+    SG_LOG( SG_GENERAL, SG_INFO, "download-dir = " << '"' << fgGetString("/sim/paths/download-dir") << '"' );
+    SG_LOG( SG_GENERAL, SG_INFO, "terrasync-dir = " << '"' << fgGetString("/sim/terrasync/scenery-dir") << '"' );
+
     SG_LOG( SG_GENERAL, SG_INFO, "aircraft-search-paths = \n\t" << simgear::strutils::join(globals->get_aircraft_paths(), "\n\t") );
     SG_LOG( SG_GENERAL, SG_INFO, "scenery-search-paths = \n\t" << simgear::strutils::join(globals->get_fg_scenery(), "\n\t") );
 }
@@ -680,7 +696,7 @@ void fgCreateSubsystems(bool duringReset) {
     // to be updated in every loop.
     // Sound manager is updated last so it can use the CPU while the GPU
     // is processing the scenery (doubled the frame-rate for me) -EMH-
-    globals->add_subsystem("sound", new FGSoundManager, SGSubsystemMgr::SOUND);
+    globals->add_new_subsystem<FGSoundManager>(SGSubsystemMgr::SOUND);
 
     ////////////////////////////////////////////////////////////////////
     // Initialize the event manager subsystem.
@@ -702,6 +718,11 @@ void fgCreateSubsystems(bool duringReset) {
     globals->add_subsystem("properties", new FGProperties);
 
 
+    ////////////////////////////////////////////////////////////////////
+    // Add the FlightGear property utilities.
+    ////////////////////////////////////////////////////////////////////
+    globals->add_new_subsystem<flightgear::AirportDynamicsManager>();
+
     ////////////////////////////////////////////////////////////////////
     // Add the performance monitoring system.
     ////////////////////////////////////////////////////////////////////
@@ -719,16 +740,11 @@ void fgCreateSubsystems(bool duringReset) {
             globals->get_props()) ) {
        throw sg_io_exception("Error loading materials file", mpath);
     }
-    
-    globals->add_subsystem( "http", new FGHTTPClient );
-    
-    ////////////////////////////////////////////////////////////////////
-    // Initialize the scenery management subsystem.
-    ////////////////////////////////////////////////////////////////////
 
-    globals->get_scenery()->get_scene_graph()
-        ->addChild(simgear::Particles::getCommonRoot());
-    simgear::GlobalParticleCallback::setSwitch(fgGetNode("/sim/rendering/particles", true));
+    // may exist already due to GUI startup
+    if (!globals->get_subsystem<FGHTTPClient>()) {
+        globals->add_new_subsystem<FGHTTPClient>();
+    }
 
     ////////////////////////////////////////////////////////////////////
     // Initialize the flight model subsystem.
@@ -742,7 +758,7 @@ void fgCreateSubsystems(bool duringReset) {
 
     // Initialize the weather modeling subsystem
     globals->add_subsystem("environment", new FGEnvironmentMgr);
-    globals->add_subsystem("ephemeris", new Ephemeris);
+    globals->add_new_subsystem<Ephemeris>();
     
     ////////////////////////////////////////////////////////////////////
     // Initialize the aircraft systems and instrumentation (before the
@@ -760,7 +776,7 @@ void fgCreateSubsystems(bool duringReset) {
 
     globals->add_subsystem( "xml-autopilot", FGXMLAutopilotGroup::createInstance("autopilot"), SGSubsystemMgr::FDM );
     globals->add_subsystem( "xml-proprules", FGXMLAutopilotGroup::createInstance("property-rule"), SGSubsystemMgr::GENERAL );
-    globals->add_subsystem( "route-manager", new FGRouteMgr );
+    globals->add_new_subsystem<FGRouteMgr>();
 
     ////////////////////////////////////////////////////////////////////
     // Initialize the Input-Output subsystem
@@ -796,6 +812,8 @@ void fgCreateSubsystems(bool duringReset) {
     ////////////////////////////////////////////////////////////////////
    // Initialize the ATC subsystem
     ////////////////////////////////////////////////////////////////////
+
+    globals->add_new_subsystem<PerformanceDB>(SGSubsystemMgr::POST_FDM);
     globals->add_subsystem("ATC", new FGATCManager, SGSubsystemMgr::POST_FDM);
 
     ////////////////////////////////////////////////////////////////////
@@ -807,7 +825,7 @@ void fgCreateSubsystems(bool duringReset) {
     ////////////////////////////////////////////////////////////////////
     // Initialise the AI Model Manager
     ////////////////////////////////////////////////////////////////////
-    SG_LOG(SG_GENERAL, SG_INFO, "  AI Model Manager");
+
     globals->add_subsystem("ai-model", new FGAIManager, SGSubsystemMgr::POST_FDM);
     globals->add_subsystem("submodel-mgr", new FGSubmodelMgr, SGSubsystemMgr::POST_FDM);
 
@@ -830,7 +848,7 @@ void fgCreateSubsystems(bool duringReset) {
     // Initialize the controls subsystem.
     ////////////////////////////////////////////////////////////////////
     
-    globals->add_subsystem("controls", new FGControls, SGSubsystemMgr::GENERAL);
+    globals->add_new_subsystem<FGControls>(SGSubsystemMgr::GENERAL);
 
     ////////////////////////////////////////////////////////////////////
     // Initialize the input subsystem.
@@ -875,13 +893,10 @@ void fgCreateSubsystems(bool duringReset) {
         globals->add_subsystem("events", globals->get_event_mgr(), SGSubsystemMgr::DISPLAY);
     }
 
-    globals->add_subsystem("aircraft-model", new FGAircraftModel, SGSubsystemMgr::DISPLAY);
-    globals->add_subsystem("model-manager", new FGModelMgr, SGSubsystemMgr::DISPLAY);
+    globals->add_new_subsystem<FGAircraftModel>(SGSubsystemMgr::DISPLAY);
+    globals->add_new_subsystem<FGModelMgr>(SGSubsystemMgr::DISPLAY);
 
-    globals->add_subsystem("view-manager", new FGViewMgr, SGSubsystemMgr::DISPLAY);
-
-    globals->add_subsystem("tile-manager", globals->get_tile_mgr(), 
-      SGSubsystemMgr::DISPLAY);
+    globals->add_new_subsystem<FGViewMgr>(SGSubsystemMgr::DISPLAY);
 }
 
 void fgPostInitSubsystems()
@@ -992,7 +1007,7 @@ void fgStartNewReset()
     
     SGSubsystemMgr* subsystemManger = globals->get_subsystem_mgr();
     // Nasal is manually inited in fgPostInit, ensure it's already shutdown
-    // before other subsystems, so Nasal listeners don't fire during shutdonw
+    // before other subsystems, so Nasal listeners don't fire during shutdown
     SGSubsystem* nasal = subsystemManger->get_subsystem("nasal");
     nasal->shutdown();
     nasal->unbind();
@@ -1008,7 +1023,7 @@ void fgStartNewReset()
         string_list::const_iterator it;
         for (it = names.begin(); it != names.end(); ++it) {
             if ((*it == "time") || (*it == "terrasync") || (*it == "events")
-                || (*it == "lighting"))
+                || (*it == "lighting") || (*it == FGTileMgr::subsystemName()) || (*it == FGScenery::subsystemName()))
             {
                 continue;
             }
@@ -1031,8 +1046,9 @@ void fgStartNewReset()
     
     // order is important here since tile-manager shutdown needs to
     // access the scenery object
-    globals->set_tile_mgr(NULL);
-    globals->set_scenery(NULL);
+    subsystemManger->remove(FGTileMgr::subsystemName());
+    subsystemManger->remove(FGScenery::subsystemName());
+
     FGScenery::getPagerSingleton()->clearRequests();
     flightgear::CameraGroup::setDefault(NULL);
     
@@ -1042,14 +1058,15 @@ void fgStartNewReset()
     render->getViewer()->getDatabasePager()->clear();
     
     osgDB::Registry::instance()->clearObjectCache();
-    
+    // Pager requests depend on this, so don't clear it until now
+    sgUserDataInit( NULL );
+
     // preserve the event handler; re-creating it would entail fixing the
     // idle handler
     osg::ref_ptr<flightgear::FGEventHandler> eventHandler = render->getEventHandler();
     
     globals->set_renderer(NULL);
     globals->set_matlib(NULL);
-    globals->set_chatter_queue(NULL);
     
     simgear::clearEffectCache();
     simgear::SGModelLib::resetPropertyRoot();
@@ -1084,6 +1101,8 @@ void fgStartNewReset()
     render->init();
     render->setViewer(viewer.get());
 
+    sgUserDataInit( globals->get_props() );
+
     viewer->getDatabasePager()->setUpThreads(1, 1);
     
     // must do this before splashinit for Rembrandt
@@ -1110,3 +1129,24 @@ void fgStartNewReset()
     fgSetBool("/sim/sceneryloaded",false);
 }
 
+void fgInitPackageRoot()
+{
+    if (globals->packageRoot()) {
+        return;
+    }
+
+    SGPath packageAircraftDir = flightgear::Options::sharedInstance()->valueForOption("download-dir");
+    if (packageAircraftDir.isNull()) {
+        packageAircraftDir = flightgear::defaultDownloadDir();
+    }
+
+    packageAircraftDir.append("Aircraft");
+
+    SG_LOG(SG_GENERAL, SG_INFO, "init package root at:" << packageAircraftDir.str());
+
+
+    SGSharedPtr<Root> pkgRoot(new Root(packageAircraftDir, FLIGHTGEAR_VERSION));
+    // set the http client later (too early in startup right now)
+    globals->setPackageRoot(pkgRoot);
+
+}
