@@ -48,15 +48,12 @@
 #include <Aircraft/controls.hxx>
 #include <Airports/runways.hxx>
 #include <Autopilot/route_mgr.hxx>
-#include <GUI/FGFontCache.hxx>
 #include <GUI/gui.h>
-#include <MultiPlayer/multiplaymgr.hxx>
 #include <Scenery/scenery.hxx>
 #include <Scenery/tilemgr.hxx>
 #include <Navaids/navlist.hxx>
 #include <Viewer/renderer.hxx>
 #include <Viewer/viewmgr.hxx>
-#include <Sound/sample_queue.hxx>
 
 #include "globals.hxx"
 #include "locale.hxx"
@@ -71,29 +68,44 @@ public:
     simgear::ResourceProvider(simgear::ResourceManager::PRIORITY_HIGH)
   {
   }
-  
+
   virtual SGPath resolve(const std::string& aResource, SGPath&) const
   {
     string_list pieces(sgPathBranchSplit(aResource));
     if ((pieces.size() < 3) || (pieces.front() != "Aircraft")) {
       return SGPath(); // not an Aircraft path
     }
-    
+
   // test against the aircraft-dir property
     const char* aircraftDir = fgGetString("/sim/aircraft-dir");
     string_list aircraftDirPieces(sgPathBranchSplit(aircraftDir));
     if (!aircraftDirPieces.empty() && (aircraftDirPieces.back() == pieces[1])) {
+        // disable this warning for release builds
+#if defined(ENABLE_DEV_WARNINGS)
+        SGPath betterPath;
+        for (unsigned int i=2; i<pieces.size(); ++i) {
+            betterPath.append(pieces[i]);
+        }
+        SG_LOG(SG_AIRCRAFT, SG_WARN, "resolved path:" << aResource << " using /sim/aircraft-dir.\n"
+               "This is legacy behaviour, change to an aircraft relative path:" << betterPath);
+#endif
+
         // current aircraft-dir matches resource aircraft
         SGPath r(aircraftDir);
         for (unsigned int i=2; i<pieces.size(); ++i) {
           r.append(pieces[i]);
         }
-        
+
         if (r.exists()) {
           return r;
+        } else {
+          // Stop here, otherwise we could end up returning a resource that
+          // belongs to an unrelated version of the same aircraft (from a
+          // different aircraft directory).
+          return SGPath();
         }
     }
-  
+
   // try each aircraft dir in turn
     std::string res(aResource, 9); // resource path with 'Aircraft/' removed
     const string_list& dirs(globals->get_aircraft_paths());
@@ -104,7 +116,7 @@ public:
         return p;
       }
     } // of aircraft path iteration
-    
+
     return SGPath(); // not found
   }
 };
@@ -116,7 +128,7 @@ public:
     simgear::ResourceProvider(simgear::ResourceManager::PRIORITY_HIGH)
   {
   }
-  
+
   virtual SGPath resolve(const std::string& aResource, SGPath&) const
   {
     const char* aircraftDir = fgGetString("/sim/aircraft-dir");
@@ -143,22 +155,16 @@ FGGlobals::FGGlobals() :
     fg_root( "" ),
     fg_home( "" ),
     time_params( NULL ),
-    ephem( NULL ),
-    route_mgr( NULL ),
-    controls( NULL ),
-    viewmgr( NULL ),
     commands( SGCommandMgr::instance() ),
     channel_options_list( NULL ),
     initial_waypoints( NULL ),
-    fontcache ( new FGFontCache ),
     channellist( NULL ),
-    haveUserSettings(false),
-    _chatter_queue(NULL)
+    haveUserSettings(false)
 {
     SGPropertyNode* root = new SGPropertyNode;
     props = SGPropertyNode_ptr(root);
     locale = new FGLocale(props);
-    
+
     simgear::ResourceManager::instance()->addProvider(new AircraftResourceProvider);
     simgear::ResourceManager::instance()->addProvider(new CurrentAircraftDirProvider);
     initProperties();
@@ -167,15 +173,15 @@ FGGlobals::FGGlobals() :
 void FGGlobals::initProperties()
 {
     simgear::PropertyObjectBase::setDefaultRoot(props);
-    
+
     positionLon = props->getNode("position/longitude-deg", true);
     positionLat = props->getNode("position/latitude-deg", true);
     positionAlt = props->getNode("position/altitude-ft", true);
-    
+
     viewLon = props->getNode("sim/current-view/viewer-lon-deg", true);
     viewLat = props->getNode("sim/current-view/viewer-lat-deg", true);
     viewAlt = props->getNode("sim/current-view/viewer-elev-ft", true);
-    
+
     orientPitch = props->getNode("orientation/pitch-deg", true);
     orientHeading = props->getNode("orientation/heading-deg", true);
     orientRoll = props->getNode("orientation/roll-deg", true);
@@ -183,31 +189,13 @@ void FGGlobals::initProperties()
 }
 
 // Destructor
-FGGlobals::~FGGlobals() 
+FGGlobals::~FGGlobals()
 {
     // save user settings (unless already saved)
     saveUserSettings();
 
-    // The AIModels manager performs a number of actions upon
-    // Shutdown that implicitly assume that other subsystems
-    // are still operational (Due to the dynamic allocation and
-    // deallocation of AIModel objects. To ensure we can safely
-    // shut down all subsystems, make sure we take down the 
-    // AIModels system first.
-    SGSubsystemRef ai = subsystem_mgr->get_subsystem("ai-model");
-    if (ai) {
-        subsystem_mgr->remove("ai-model");
-        ai->unbind();
-        ai.clear(); // ensure AI is deleted now, not at end of this method
-    }
-    
-    subsystem_mgr->shutdown();
-    subsystem_mgr->unbind();    
-
-    subsystem_mgr->remove("aircraft-model");
-    subsystem_mgr->remove("tile-manager");
-    subsystem_mgr->remove("model-manager");
-    _tile_mgr.clear();
+    // stop OSG threading first, to avoid thread races while we tear down
+    // scene-graph pieces
 
     osg::ref_ptr<osgViewer::Viewer> vw(renderer->getViewer());
     if (vw) {
@@ -217,43 +205,45 @@ FGGlobals::~FGGlobals()
         // GraphicsContext)
         vw->stopThreading();
     }
-    
+
+    subsystem_mgr->shutdown();
+    subsystem_mgr->unbind();
+
+    subsystem_mgr->remove(FGTileMgr::subsystemName());
     // don't cancel the pager until after shutdown, since AIModels (and
     // potentially others) can queue delete requests on the pager.
     if (vw && vw->getDatabasePager()) {
         vw->getDatabasePager()->cancel();
         vw->getDatabasePager()->clear();
     }
-    
+
     osgDB::Registry::instance()->clearObjectCache();
-    
+    subsystem_mgr->remove(FGScenery::subsystemName());
+
     // renderer touches subsystems during its destruction
     set_renderer(NULL);
-    _scenery.clear();
-    _chatter_queue.clear();
-    
+
     delete subsystem_mgr;
     subsystem_mgr = NULL; // important so ::get_subsystem returns NULL
     vw = 0; // don't delete the viewer until now
 
     delete time_params;
     set_matlib(NULL);
-    delete route_mgr;
+
     delete channel_options_list;
     delete initial_waypoints;
-    delete fontcache;
     delete channellist;
 
     simgear::PropertyObjectBase::setDefaultRoot(NULL);
     simgear::SGModelLib::resetPropertyRoot();
-    
+
     delete locale;
     locale = NULL;
-    
+
     cleanupListeners();
-    
+
     props.clear();
-    
+
     delete commands;
 }
 
@@ -273,14 +263,15 @@ void FGGlobals::set_fg_root (const std::string &root) {
                 << fg_root << "'\n***\n***");
     }
 
-    // remove /sim/fg-root before writing to prevent hijacking
+    // deliberately not a tied property, for fgValidatePath security
+    // write-protect to avoid accidents
     SGPropertyNode *n = fgGetNode("/sim", true);
     n->removeChild("fg-root", 0);
     n = n->getChild("fg-root", 0, true);
     n->setStringValue(fg_root.c_str());
     n->setAttribute(SGPropertyNode::WRITE, false);
-    
-    simgear::ResourceManager::instance()->addBasePath(fg_root, 
+
+    simgear::ResourceManager::instance()->addBasePath(fg_root,
       simgear::ResourceManager::PRIORITY_DEFAULT);
 }
 
@@ -315,7 +306,7 @@ void FGGlobals::append_data_path(const SGPath& path)
     if (!path.exists()) {
         SG_LOG(SG_GENERAL, SG_WARN, "adding non-existant data path:" << path);
     }
-    
+
     additional_data_paths.push_back(path);
 }
 
@@ -327,27 +318,27 @@ SGPath FGGlobals::find_data_dir(const std::string& pathSuffix) const
             return p;
         }
     }
-    
+
     SGPath rootPath(fg_root);
     rootPath.append(pathSuffix);
     if (rootPath.exists()) {
         return rootPath;
     }
-    
+
     SG_LOG(SG_GENERAL, SG_WARN, "dir not found in any data path:" << pathSuffix);
     return SGPath();
 }
 
-void FGGlobals::append_fg_scenery (const std::string &paths)
+void FGGlobals::append_fg_scenery (const std::string &paths, bool secure)
 {
     SGPropertyNode* sim = fgGetNode("/sim", true);
 
   // find first unused fg-scenery property in /sim
     int propIndex = 0;
     while (sim->getChild("fg-scenery", propIndex) != NULL) {
-      ++propIndex; 
+      ++propIndex;
     }
-  
+
     BOOST_FOREACH(const SGPath& path, sgPathSplit( paths )) {
         SGPath abspath(path.realpath());
         if (!abspath.exists()) {
@@ -361,35 +352,43 @@ void FGGlobals::append_fg_scenery (const std::string &paths)
         SG_LOG(SG_GENERAL, SG_INFO, "skipping duplicate add of scenery path:" << abspath.str());
         continue;
       }
-      
+
+      // tell the ResouceManager about the scenery path
+      // needed to load Models from this scenery path
+      simgear::ResourceManager::instance()->addBasePath(abspath.str(),
+        simgear::ResourceManager::PRIORITY_DEFAULT);
+
         simgear::Dir dir(abspath);
         SGPath terrainDir(dir.file("Terrain"));
         SGPath objectsDir(dir.file("Objects"));
-        
-      // this code used to add *either* the base dir, OR add the 
+
+      // this code used to add *either* the base dir, OR add the
       // Terrain and Objects subdirs, but the conditional logic was commented
       // out, such that all three dirs are added. Unfortunately there's
       // no information as to why the change was made.
         fg_scenery.push_back(abspath.str());
-        
+        if (secure) {
+          secure_fg_scenery.push_back(abspath.str());
+        }
+
         if (terrainDir.exists()) {
           fg_scenery.push_back(terrainDir.str());
         }
-        
+
         if (objectsDir.exists()) {
           fg_scenery.push_back(objectsDir.str());
         }
-        
+
         // insert a marker for FGTileEntry::load(), so that
         // FG_SCENERY=A:B becomes list ["A/Terrain", "A/Objects", "",
         // "B/Terrain", "B/Objects", ""]
         fg_scenery.push_back("");
-        
+
       // make scenery dirs available to Nasal
         SGPropertyNode* n = sim->getChild("fg-scenery", propIndex++, true);
         n->setStringValue(abspath.str());
         n->setAttribute(SGPropertyNode::WRITE, false);
-        
+
         // temporary fix so these values survive reset
         n->setAttribute(SGPropertyNode::PRESERVE, true);
     } // of path list iteration
@@ -398,6 +397,9 @@ void FGGlobals::append_fg_scenery (const std::string &paths)
 void FGGlobals::clear_fg_scenery()
 {
   fg_scenery.clear();
+  secure_fg_scenery.clear();
+  fgGetNode("/sim", true)->removeChildren("fg-scenery");
+
 }
 
 void FGGlobals::set_catalog_aircraft_path(const SGPath& path)
@@ -475,7 +477,7 @@ void FGGlobals::set_renderer(FGRenderer *render)
     if (render == renderer) {
         return;
     }
-    
+
     delete renderer;
     renderer = render;
 }
@@ -487,12 +489,12 @@ FGGlobals::get_subsystem_mgr () const
 }
 
 SGSubsystem *
-FGGlobals::get_subsystem (const char * name)
+FGGlobals::get_subsystem (const char * name) const
 {
     if (!subsystem_mgr) {
         return NULL;
     }
-    
+
     return subsystem_mgr->get_subsystem(name);
 }
 
@@ -505,15 +507,6 @@ FGGlobals::add_subsystem (const char * name,
     subsystem_mgr->add(name, subsystem, type, min_time_sec);
 }
 
-SGSoundMgr *
-FGGlobals::get_soundmgr () const
-{
-    if (subsystem_mgr)
-        return (SGSoundMgr*) subsystem_mgr->get_subsystem("sound");
-
-    return NULL;
-}
-
 SGEventMgr *
 FGGlobals::get_event_mgr () const
 {
@@ -562,7 +555,7 @@ static void treeDumpRefCounts(int depth, SGPropertyNode* nd)
         if (SGReferenced::count(cp) > 1) {
             SG_LOG(SG_GENERAL, SG_INFO, "\t" << cp->getPath() << " refcount:" << SGReferenced::count(cp));
         }
-        
+
         treeDumpRefCounts(depth + 1, cp);
     }
 }
@@ -572,7 +565,7 @@ static void treeClearAliases(SGPropertyNode* nd)
     if (nd->isAlias()) {
         nd->unalias();
     }
-    
+
     for (int i=0; i<nd->nChildren(); ++i) {
         SGPropertyNode* cp = nd->getChild(i);
         treeClearAliases(cp);
@@ -583,9 +576,9 @@ void
 FGGlobals::resetPropertyRoot()
 {
     delete locale;
-    
+
     cleanupListeners();
-    
+
     // we don't strictly need to clear these (they will be reset when we
     // initProperties again), but trying to reduce false-positives when dumping
     // ref-counts.
@@ -598,19 +591,19 @@ FGGlobals::resetPropertyRoot()
     orientPitch.clear();
     orientHeading.clear();
     orientRoll.clear();
-    
+
     // clear aliases so ref-counts are accurate when dumped
     treeClearAliases(props);
-    
+
     SG_LOG(SG_GENERAL, SG_INFO, "root props refcount:" << props.getNumRefs());
     treeDumpRefCounts(0, props);
 
     //BaseStackSnapshot::dumpAll(std::cout);
-    
+
     props = new SGPropertyNode;
     initProperties();
     locale = new FGLocale(props);
-    
+
     // remove /sim/fg-root before writing to prevent hijacking
     SGPropertyNode *n = props->getNode("/sim", true);
     n->removeChild("fg-root", 0);
@@ -626,7 +619,7 @@ static std::string autosaveName()
     if (versionParts.size() < 2) {
         return "autosave.xml";
     }
-    
+
     os << "autosave_" << versionParts[0] << "_" << versionParts[1] << ".xml";
     return os.str();
 }
@@ -678,12 +671,6 @@ FGGlobals::saveUserSettings()
     }
 }
 
-FGViewer *
-FGGlobals::get_current_view () const
-{
-  return viewmgr->get_current_view();
-}
-
 long int FGGlobals::get_warp() const
 {
   return fgGetInt("/sim/time/warp");
@@ -706,22 +693,23 @@ void FGGlobals::set_warp_delta( long int d )
 
 FGScenery* FGGlobals::get_scenery () const
 {
-    return _scenery.get();
-}
-
-void FGGlobals::set_scenery ( FGScenery *s )
-{
-    _scenery = s;
+    return get_subsystem<FGScenery>();
 }
 
 FGTileMgr* FGGlobals::get_tile_mgr () const
 {
-    return _tile_mgr.get();
+    return get_subsystem<FGTileMgr>();
 }
 
-void FGGlobals::set_tile_mgr ( FGTileMgr *t )
+FGViewMgr *FGGlobals::get_viewmgr() const
 {
-    _tile_mgr = t;
+    return get_subsystem<FGViewMgr>();
+}
+
+flightgear::View* FGGlobals::get_current_view () const
+{
+    FGViewMgr* vm = get_viewmgr();
+    return vm ? vm->get_current_view() : 0;
 }
 
 void FGGlobals::set_matlib( SGMaterialLib *m )
@@ -729,14 +717,9 @@ void FGGlobals::set_matlib( SGMaterialLib *m )
     matlib = m;
 }
 
-FGSampleQueue* FGGlobals::get_chatter_queue() const
+FGControls *FGGlobals::get_controls() const
 {
-    return _chatter_queue;
-}
-
-void FGGlobals::set_chatter_queue(FGSampleQueue* queue)
-{
-    _chatter_queue = queue;
+    return get_subsystem<FGControls>();
 }
 
 void FGGlobals::addListenerToCleanup(SGPropertyChangeListener* l)
