@@ -26,6 +26,7 @@
 #endif
 
 #include <simgear/compiler.h>
+#include <simgear/props/props_io.hxx>
 
 #include <iostream>
 
@@ -54,6 +55,7 @@ extern bool global_crashRptEnabled;
 #include <simgear/math/SGMath.hxx>
 #include <simgear/math/sg_random.h>
 #include <simgear/misc/strutils.hxx>
+#include <simgear/scene/tgdb/GroundLightManager.hxx>
 
 #include <Model/panelnode.hxx>
 #include <Scenery/scenery.hxx>
@@ -67,7 +69,6 @@ extern bool global_crashRptEnabled;
 #include <Viewer/WindowSystemAdapter.hxx>
 #include <Navaids/NavDataCache.hxx>
 #include <Include/version.h>
-#include <Viewer/WindowBuilder.hxx>
 
 #include "fg_commands.hxx"
 #include "fg_io.hxx"
@@ -82,7 +83,6 @@ extern bool global_crashRptEnabled;
 #include "options.hxx"
 
 #if defined(HAVE_QT)
-#include <QApplication>
 #include <GUI/QtLauncher.hxx>
 #endif
 
@@ -128,7 +128,7 @@ static void initTerrasync()
     
     // start TerraSync up now, so it can be synchronizing shared models
     // and airports data in parallel with a nav-cache rebuild.
-    SGPath tsyncCache(globals->get_fg_home());
+    SGPath tsyncCache(terraSyncDir);
     tsyncCache.append("terrasync-cache.xml");
     
     // wipe the cache file if requested
@@ -150,6 +150,46 @@ static void initTerrasync()
     terra_sync->init();
 }
 
+static void fgSetVideoOptions()
+{
+    std::string vendor = fgGetString("/sim/rendering/gl-vendor");
+    SGPath path(globals->get_fg_root());
+    path.append("Video");
+    path.append(vendor);
+    if (path.exists())
+    {
+        std::string renderer = fgGetString("/sim/rendering/gl-renderer");
+        size_t pos = renderer.find("x86/");
+        if (pos == std::string::npos) {
+            pos = renderer.find('/');
+        }
+        if (pos == std::string::npos) {
+            pos = renderer.find(" (");
+        }
+        if (pos != std::string::npos) {
+            renderer = renderer.substr(0, pos);
+        }
+        path.append(renderer+".xml");
+        if (path.exists()) {
+            SG_LOG(SG_INPUT, SG_INFO, "Reading video settings from " << path.str());
+            try {
+                SGPropertyNode *r_prop = fgGetNode("/sim/rendering");
+                readProperties(path.str(), r_prop);
+            } catch (sg_exception& e) {
+                SG_LOG(SG_INPUT, SG_WARN, "failed to read video settings:" << e.getMessage()
+                << "(from " << e.getOrigin() << ")");
+            }
+
+            flightgear::Options* options = flightgear::Options::sharedInstance();
+            if (!options->isOptionSet("ignore-autosave"))
+            {
+                SGPath dataPath = globals->get_fg_home();
+                globals->loadUserSettings(dataPath);
+            }
+        }
+    }
+}
+
 static void checkOpenGLVersion()
 {
 #if defined(SG_MAC)
@@ -207,6 +247,7 @@ static void fgIdleFunction ( void ) {
         if (guiInit())
         {
             checkOpenGLVersion();
+            fgSetVideoOptions();
             idle_state+=2;
             fgSplashProgress("loading-aircraft-list");
         }
@@ -222,10 +263,7 @@ static void fgIdleFunction ( void ) {
         if (done) {
           ++idle_state;
           fgSplashProgress("init-scenery");
-        } else {
-          fgSplashProgress("loading-nav-dat");
         }
-      
     } else if ( idle_state == 4 ) {
         idle_state++;
 
@@ -264,10 +302,10 @@ static void fgIdleFunction ( void ) {
         // Initialize the TG scenery subsystem.
         ////////////////////////////////////////////////////////////////////
         
-        globals->set_scenery( new FGScenery );
+        globals->add_new_subsystem<FGScenery>(SGSubsystemMgr::DISPLAY);
         globals->get_scenery()->init();
         globals->get_scenery()->bind();
-        globals->set_tile_mgr( new FGTileMgr );
+        globals->add_new_subsystem<FGTileMgr>(SGSubsystemMgr::DISPLAY);
         
         fgSplashProgress("creating-subsystems");
     } else if (( idle_state == 7 ) || (idle_state == 2007)) {
@@ -414,12 +452,7 @@ int fgMainInit( int argc, char **argv )
         logToFile();
     }
     
-    std::string version;
-#ifdef FLIGHTGEAR_VERSION
-    version = FLIGHTGEAR_VERSION;
-#else
-    version = "unknown version";
-#endif
+    std::string version(FLIGHTGEAR_VERSION);
     SG_LOG( SG_GENERAL, SG_INFO, "FlightGear:  Version "
             << version );
     SG_LOG( SG_GENERAL, SG_INFO, "Built with " << SG_COMPILER_STR);
@@ -432,7 +465,7 @@ int fgMainInit( int argc, char **argv )
     string_list *col = new string_list;
     globals->set_channel_options_list( col );
 
-    fgValidatePath("", false);  // initialize static variables
+    fgValidatePath(globals->get_fg_home(), false);  // initialize static variables
     upper_case_property("/sim/presets/airport-id");
     upper_case_property("/sim/presets/runway");
     upper_case_property("/sim/tower/airport-id");
@@ -448,30 +481,20 @@ int fgMainInit( int argc, char **argv )
         return EXIT_SUCCESS;
     }
 
-  // launcher needs to know the aircraft paths in use
-    fgInitAircraftPaths(false);
-
 #if defined(HAVE_QT)
     bool showLauncher = flightgear::Options::checkForArg(argc, argv, "launcher");
     // an Info.plist bundle can't define command line arguments, but it can set
     // environment variables. This avoids needed a wrapper shell-script on OS-X.
     showLauncher |= (::getenv("FG_LAUNCHER") != 0);
 
-    if (showLauncher) {        
-        QApplication app(argc, argv);
-        app.setOrganizationName("FlightGear");
-        app.setApplicationName("FlightGear");
-        app.setOrganizationDomain("flightgear.org");
-    
-        // avoid double Apple menu and other weirdness if both Qt and OSG
-        // try to initialise various Cocoa structures.
-        flightgear::WindowBuilder::setPoseAsStandaloneApp(false);
-
-        if (!QtLauncher::runLauncherDialog()) {
+    if (showLauncher) {
+        flightgear::initApp(argc, argv);
+        if (!flightgear::runLauncherDialog()) {
             return EXIT_SUCCESS;
         }
     }
 #endif
+    fgInitAircraftPaths(false);
 
     configResult = fgInitAircraft(false);
     if (configResult == flightgear::FG_OPTIONS_ERROR) {
@@ -529,6 +552,9 @@ int fgMainInit( int argc, char **argv )
     // delete the NavCache here. This will cause the destruction of many cached
     // objects (eg, airports, navaids, runways).
     delete flightgear::NavDataCache::instance();
+    simgear::GroundLightManager::instance()->getRunwayLightStateSet()->clear();
+    simgear::GroundLightManager::instance()->getTaxiLightStateSet()->clear();
+    simgear::GroundLightManager::instance()->getGroundLightStateSet()->clear();
   
     return result;
 }
