@@ -183,6 +183,10 @@ static bool setPosFromAirportIDandHdg( const string& id, double tgt_hdg ) {
 // Set current_options lon/lat given an airport id and parkig position name
 static bool fgSetPosFromAirportIDandParkpos( const string& id, const string& parkpos )
 {
+  string fltType;
+  string acOperator;
+  string acType; // Currently not used by findAvailable parking, so safe to leave empty.
+  SGPath acData;
   if ( id.empty() )
     return false;
   
@@ -192,7 +196,7 @@ static bool fgSetPosFromAirportIDandParkpos( const string& id, const string& par
     SG_LOG( SG_GENERAL, SG_ALERT, "Failed to find airport " << id );
     return false;
   }
-  FGAirportDynamics* dcs = apt->getDynamics();
+  FGAirportDynamicsRef dcs = apt->getDynamics();
   if (!dcs) {
     SG_LOG( SG_GENERAL, SG_ALERT,
            "Airport " << id << "does not appear to have parking information available");
@@ -202,9 +206,7 @@ static bool fgSetPosFromAirportIDandParkpos( const string& id, const string& par
   ParkingAssignment pka;
   double radius = fgGetDouble("/sim/dimensions/radius-m");
   if ((parkpos == string("AVAILABLE")) && (radius > 0)) {
-    string fltType;
-    string acOperator;
-    SGPath acData;
+    
     try {
       acData = globals->get_fg_home();
       acData.append("aircraft-data");
@@ -232,9 +234,10 @@ static bool fgSetPosFromAirportIDandParkpos( const string& id, const string& par
       acOperator = fgGetString("/sim/aircraft-operator"   );
     }
     
-    string acType; // Currently not used by findAvailable parking, so safe to leave empty.
+    
     pka = dcs->getAvailableParking(radius, fltType, acType, acOperator);
     if (pka.isValid()) {
+        // why is the following line necessary?
       fgGetString("/sim/presets/parkpos");
       fgSetString("/sim/presets/parkpos", pka.parking()->getName());
     } else {
@@ -243,6 +246,7 @@ static bool fgSetPosFromAirportIDandParkpos( const string& id, const string& par
       return false;
     }
   } else {
+      
     pka = dcs->getParkingByName(parkpos);
     if (!pka.isValid()) {
       SG_LOG( SG_GENERAL, SG_ALERT,
@@ -250,7 +254,14 @@ static bool fgSetPosFromAirportIDandParkpos( const string& id, const string& par
       return false;
     }
   }
-  
+  // Why is the following line necessary?
+  fgGetString("/sim/presets/parkpos");
+  fgSetString("/sim/presets/parkpos", pka.parking()->getName());
+  // The problem is, this line doesn't work because the ParkingAssignment's refcounting mechanism:
+  // The parking will be released after this function returns. 
+  // As a temporary measure, I'll try to reserve the parking via the atc_manager, which should work, because it uses the same 
+  // mechanism as the AI traffic code. 
+  dcs->setParkingAvailable(pka.parking(), false);
   fgApplyStartOffset(pka.parking()->geod(), pka.parking()->getHeading());
   return true;
 }
@@ -342,37 +353,51 @@ static void fgSetDistOrAltFromGlideSlope() {
 
 
 // Set current_options lon/lat given an airport id and heading (degrees)
-static bool fgSetPosFromNAV( const string& id, const double& freq, FGPositioned::Type type )
+static bool fgSetPosFromNAV( const string& id,
+                             const double& freq,
+                             FGPositioned::Type type,
+                             PositionedID guid)
 {
-  FGNavList::TypeFilter filter(type);
-  const nav_list_type navlist = FGNavList::findByIdentAndFreq( id.c_str(), freq, &filter );
-  
-  if (navlist.empty()) {
-    SG_LOG( SG_GENERAL, SG_ALERT, "Failed to locate NAV = "
-           << id << ":" << freq );
-    return false;
-  }
-  
-  if( navlist.size() > 1 ) {
-    std::ostringstream buf;
-    buf << "Ambigous NAV-ID: '" << id << "'. Specify id and frequency. Available stations:" << endl;
-    for( nav_list_type::const_iterator it = navlist.begin(); it != navlist.end(); ++it ) {
-      // NDB stored in kHz, VOR stored in MHz * 100 :-P
-      double factor = (*it)->type() == FGPositioned::NDB ? 1.0 : 1/100.0;
-      string unit = (*it)->type() == FGPositioned::NDB ? "kHz" : "MHz";
-      buf << (*it)->ident() << " "
-      << std::setprecision(5) << (double)((*it)->get_freq() * factor) << " "
-      << (*it)->get_lat() << "/" << (*it)->get_lon()
-      << endl;
+    FGNavRecord* nav = 0;
+
+
+    if (guid != 0) {
+        nav = FGPositioned::loadById<FGNavRecord>(guid);
+        if (!nav)
+            return false;
+    } else {
+        FGNavList::TypeFilter filter(type);
+        const nav_list_type navlist = FGNavList::findByIdentAndFreq( id.c_str(), freq, &filter );
+
+        if (navlist.empty()) {
+          SG_LOG( SG_GENERAL, SG_ALERT, "Failed to locate NAV = "
+                 << id << ":" << freq );
+          return false;
+        }
+
+        if( navlist.size() > 1 ) {
+          std::ostringstream buf;
+          buf << "Ambigous NAV-ID: '" << id << "'. Specify id and frequency. Available stations:" << endl;
+          for( nav_list_type::const_iterator it = navlist.begin(); it != navlist.end(); ++it ) {
+            // NDB stored in kHz, VOR stored in MHz * 100 :-P
+            double factor = (*it)->type() == FGPositioned::NDB ? 1.0 : 1/100.0;
+            string unit = (*it)->type() == FGPositioned::NDB ? "kHz" : "MHz";
+            buf << (*it)->ident() << " "
+            << std::setprecision(5) << (double)((*it)->get_freq() * factor) << " "
+            << (*it)->get_lat() << "/" << (*it)->get_lon()
+            << endl;
+          }
+
+          SG_LOG( SG_GENERAL, SG_ALERT, buf.str() );
+          return false;
+        }
+
+        // nav list must be of length 1
+        nav = navlist[0];
     }
-    
-    SG_LOG( SG_GENERAL, SG_ALERT, buf.str() );
-    return false;
-  }
-  
-  FGNavRecord *nav = navlist[0];
-  fgApplyStartOffset(nav->geod(), fgGetDouble("/sim/presets/heading-deg"));
-  return true;
+
+    fgApplyStartOffset(nav->geod(), fgGetDouble("/sim/presets/heading-deg"));
+    return true;
 }
 
 // Set current_options lon/lat given an aircraft carrier id
@@ -418,11 +443,17 @@ static bool fgSetPosFromCarrier( const string& carrier, const string& posid ) {
   }
 }
 
-// Set current_options lon/lat given an airport id and heading (degrees)
-static bool fgSetPosFromFix( const string& id )
+// Set current_options lon/lat given a fix ident and GUID
+static bool fgSetPosFromFix( const string& id, PositionedID guid )
 {
-  FGPositioned::TypeFilter fixFilter(FGPositioned::FIX);
-  FGPositioned* fix = FGPositioned::findFirstWithIdent(id, &fixFilter);
+    FGPositioned* fix = NULL;
+    if (guid != 0) {
+        fix = FGPositioned::loadById<FGPositioned>(guid);
+    } else {
+        FGPositioned::TypeFilter fixFilter(FGPositioned::FIX);
+        fix = FGPositioned::findFirstWithIdent(id, &fixFilter);
+    }
+
   if (!fix) {
     SG_LOG( SG_GENERAL, SG_ALERT, "Failed to locate fix = " << id );
     return false;
@@ -480,6 +511,10 @@ bool initPosition()
   string carrier = fgGetString("/sim/presets/carrier");
   string parkpos = fgGetString("/sim/presets/parkpos");
   string fix = fgGetString("/sim/presets/fix");
+
+  // the launcher sets this to precisely identify a navaid
+    PositionedID navaidId = fgGetInt("/sim/presets/navaid-id");
+
   SGPropertyNode *hdg_preset = fgGetNode("/sim/presets/heading-deg", true);
   double hdg = hdg_preset->getDoubleValue();
   
@@ -537,14 +572,14 @@ bool initPosition()
   
   if ( !set_pos && !vor.empty() ) {
     // a VOR is requested
-    if ( fgSetPosFromNAV( vor, vor_freq, FGPositioned::VOR ) ) {
+    if ( fgSetPosFromNAV( vor, vor_freq, FGPositioned::VOR, navaidId ) ) {
       set_pos = true;
     }
   }
   
   if ( !set_pos && !ndb.empty() ) {
     // an NDB is requested
-    if ( fgSetPosFromNAV( ndb, ndb_freq, FGPositioned::NDB ) ) {
+    if ( fgSetPosFromNAV( ndb, ndb_freq, FGPositioned::NDB, navaidId ) ) {
       set_pos = true;
     }
   }
@@ -558,16 +593,26 @@ bool initPosition()
   
   if ( !set_pos && !fix.empty() ) {
     // a Fix is requested
-    if ( fgSetPosFromFix( fix ) ) {
+    if ( fgSetPosFromFix( fix, navaidId ) ) {
       set_pos = true;
     }
   }
   
   if ( !set_pos ) {
-    // No lon/lat specified, no airport specified, default to
-    // middle of KSFO field.
-    fgSetDouble("/sim/presets/longitude-deg", -122.374843);
-    fgSetDouble("/sim/presets/latitude-deg", 37.619002);
+    // No lon/lat specified, no airport specified, use the default airport
+    // TODO: don't hardcode this
+    const FGAirport* airport = fgFindAirportID("LEBL");
+    if( airport ) {
+      const SGGeod & airportGeod = airport->geod();
+      fgSetDouble("/sim/presets/longitude-deg", airportGeod.getLongitudeDeg());
+      fgSetDouble("/sim/presets/latitude-deg", airportGeod.getLatitudeDeg());
+    } else {
+      // So, the default airport is unknown? We are in serious trouble. 
+      // Let's hope KSFO still exists somehow
+      fgSetDouble("/sim/presets/longitude-deg", -122.374843);
+      fgSetDouble("/sim/presets/latitude-deg", 37.619002);
+      SG_LOG(SG_GENERAL, SG_ALERT, "Sorry, the default airport seems to be unknown.");
+    }
   }
   
   fgSetDouble( "/position/longitude-deg",
