@@ -184,6 +184,10 @@ static PositionedID readNavFromStream(std::istream& aStream,
   PositionedID navaid_dme = 0;
 
   if (type == FGPositioned::DME) {
+      // complication here: the database doesn't record TACAN sites at all,
+      // we simply infer their existence from DMEs whose name includes 'VORTAC'
+      // or 'TACAN' (since all TACAN stations include DME)
+      // hence the cache never actually includes entries of type TACAN
     FGPositioned::TypeFilter f(FGPositioned::INVALID);
     if ( name.find("VOR-DME") != std::string::npos ) {
       f.addType(FGPositioned::VOR);
@@ -194,8 +198,6 @@ static PositionedID readNavFromStream(std::istream& aStream,
       f.addType(FGPositioned::VOR);
     } else if ( name.find("NDB-DME") != std::string::npos ) {
       f.addType(FGPositioned::NDB);
-    } else if ( name.find("TACAN") != std::string::npos ) {
-      f.addType(FGPositioned::VOR);
     }
 
     if (f.maxType() > 0) {
@@ -206,9 +208,14 @@ static PositionedID readNavFromStream(std::istream& aStream,
 
         if ( simgear::strutils::uppercase(navaid_part[0]) == simgear::strutils::uppercase(dme_part[0]) ) {
           navaid_dme = ref.get()->guid();
+        } else {
+            SG_LOG(SG_NAVAID, SG_WARN, "DME " << ident << " (" << name << "), closest match has wrong name:"
+                   << ref->ident() << " (" << ref->name() << ")");
         }
+      } else {
+          SG_LOG(SG_NAVAID, SG_WARN, "Couldn't find navaid for DME:" << ident << " (" << name << ")");
       }
-    }
+    } // of have a co-located navaid to locate
   }
 
   if ((type >= FGPositioned::ILS) && (type <= FGPositioned::GS)) {
@@ -251,6 +258,9 @@ static PositionedID readNavFromStream(std::istream& aStream,
   
   return r;
 }
+
+// generated by wc -l on uncomrpessed nav.dat
+const unsigned int LINES_IN_NAV_DAT = 26775;
   
 // load and initialize the navigational databases
 bool navDBInit(const SGPath& path)
@@ -260,17 +270,27 @@ bool navDBInit(const SGPath& path)
         SG_LOG( SG_NAVAID, SG_ALERT, "Cannot open file: " << path.str() );
       return false;
     }
-  
+
   autoAlignLocalizers = fgGetBool("/sim/navdb/localizers/auto-align", true);
   autoAlignThreshold = fgGetDouble( "/sim/navdb/localizers/auto-align-threshold-deg", 5.0 );
+    NavDataCache* cache = NavDataCache::instance();
 
     // skip first two lines
     in >> skipeol;
     in >> skipeol;
+    unsigned int lineNumber = 2;
 
     while (!in.eof()) {
       readNavFromStream(in);
       in >> skipcomment;
+
+        ++lineNumber;
+        if ((lineNumber % 100) == 0) {
+            // every 100 lines
+            unsigned int percent = (lineNumber * 100) / LINES_IN_NAV_DAT;
+            cache->setRebuildPhaseProgress(NavDataCache::REBUILD_NAVAIDS, percent);
+        }
+
     } // of stream data loop
   
   return true;
@@ -279,7 +299,7 @@ bool navDBInit(const SGPath& path)
   
 bool loadCarrierNav(const SGPath& path)
 {    
-    SG_LOG( SG_NAVAID, SG_INFO, "opening file: " << path.str() );    
+    SG_LOG( SG_NAVAID, SG_DEBUG, "opening file: " << path.str() );
     sg_gzifstream incarrier( path.str() );
     
     if ( !incarrier.is_open() ) {
@@ -298,7 +318,7 @@ bool loadCarrierNav(const SGPath& path)
   
 bool loadTacan(const SGPath& path, FGTACANList *channellist)
 {
-    SG_LOG( SG_NAVAID, SG_INFO, "opening file: " << path.str() );
+    SG_LOG( SG_NAVAID, SG_DEBUG, "opening file: " << path.str() );
     sg_gzifstream inchannel( path.str() );
     
     if ( !inchannel.is_open() ) {
