@@ -47,11 +47,8 @@ typedef nasal::Ghost<pkg::PackageRef> NasalPackage;
 typedef nasal::Ghost<pkg::CatalogRef> NasalCatalog;
 typedef nasal::Ghost<pkg::InstallRef> NasalInstall;
 
-#define ENABLE_PACKAGE_SYSTEM 1
 
-namespace {
-  
-  class FGDelegate : public pkg::Delegate
+class FGHTTPClient::FGDelegate : public pkg::Delegate
 {
 public:
   virtual void refreshComplete()
@@ -69,50 +66,55 @@ public:
       r->scheduleToUpdate((*it)->install());
     }
   }
-
-  virtual void failedRefresh(pkg::Catalog* aCat, FailureCode aReason)
+    
+  virtual void catalogRefreshed(pkg::CatalogRef aCat, StatusCode aReason)
   {
+      if (aCat.ptr() == NULL) {
+          SG_LOG(SG_IO, SG_INFO, "refresh of all catalogs done");
+          return;
+      }
+      
     switch (aReason) {
-    case pkg::Delegate::FAIL_SUCCESS:
-        SG_LOG(SG_IO, SG_WARN, "refresh of Catalog done");
+    case pkg::Delegate::STATUS_SUCCESS:
+    case pkg::Delegate::STATUS_REFRESHED:
+        SG_LOG(SG_IO, SG_INFO, "refresh of Catalog done:" << aCat->url());
         break;
-        
+            
+    case pkg::Delegate::STATUS_IN_PROGRESS:
+        SG_LOG(SG_IO, SG_INFO, "refresh of Catalog started:" << aCat->url());
+        break;
+            
     default:
         SG_LOG(SG_IO, SG_WARN, "refresh of Catalog " << aCat->url() << " failed:" << aReason);
     }
   }
   
-  virtual void startInstall(pkg::Install* aInstall)
+  virtual void startInstall(pkg::InstallRef aInstall)
   {
     SG_LOG(SG_IO, SG_INFO, "beginning install of:" << aInstall->package()->id()
            << " to local path:" << aInstall->path());
 
   }
   
-  virtual void installProgress(pkg::Install* aInstall, unsigned int aBytes, unsigned int aTotal)
+  virtual void installProgress(pkg::InstallRef aInstall, unsigned int aBytes, unsigned int aTotal)
   {
-    SG_LOG(SG_IO, SG_INFO, "installing:" << aInstall->package()->id() << ":"
-           << aBytes << " of " << aTotal);
   }
   
-  virtual void finishInstall(pkg::Install* aInstall)
+  virtual void finishInstall(pkg::InstallRef aInstall, StatusCode aReason)
   {
+      if (aReason == STATUS_SUCCESS) {
     SG_LOG(SG_IO, SG_INFO, "finished install of:" << aInstall->package()->id()
            << " to local path:" << aInstall->path());
+      } else {
+          SG_LOG(SG_IO, SG_WARN, "install failed of:" << aInstall->package()->id()
+                 << " to local path:" << aInstall->path());
+      }
 
   }
-  
-  virtual void failedInstall(pkg::Install* aInstall, FailureCode aReason)
-  {
-    SG_LOG(SG_IO, SG_WARN, "install failed of:" << aInstall->package()->id()
-           << " to local path:" << aInstall->path());
-  }
+}; // of FGHTTPClient::FGDelegate
 
-};
-
-} // of anonymous namespace
-
-FGHTTPClient::FGHTTPClient()
+FGHTTPClient::FGHTTPClient() :
+    _inited(false)
 {
 }
 
@@ -122,6 +124,12 @@ FGHTTPClient::~FGHTTPClient()
 
 void FGHTTPClient::init()
 {
+    // launcher may need to setup HTTP access abnormally early, so
+    // guard against duplicate inits
+    if (_inited) {
+        return;
+    }
+
   _http.reset(new simgear::HTTP::Client);
   
   std::string proxyHost(fgGetString("/sim/presets/proxy/host"));
@@ -132,27 +140,63 @@ void FGHTTPClient::init()
     _http->setProxy(proxyHost, proxyPort, proxyAuth);
   }
   
-#ifdef ENABLE_PACKAGE_SYSTEM
   pkg::Root* packageRoot = globals->packageRoot();
   if (packageRoot) {
     // package system needs access to the HTTP engine too
     packageRoot->setHTTPClient(_http.get());
     
-    packageRoot->setDelegate(new FGDelegate);
-    
-    // setup default catalog if not present
-    pkg::Catalog* defaultCatalog = packageRoot->getCatalogById("org.flightgear.default");
-    if (!defaultCatalog) {
-      // always show this message
-      SG_LOG(SG_GENERAL, SG_ALERT, "default catalog not found, installing...");
-      pkg::Catalog::createFromUrl(packageRoot,
-          "http://fgfs.goneabitbursar.com/pkg/" FLIGHTGEAR_VERSION "/default-catalog.xml");
-    }
+    _packageDelegate.reset(new FGDelegate);
+    packageRoot->addDelegate(_packageDelegate.get());
     
     // start a refresh now
-    packageRoot->refresh();
+    // setting 'force' true to work around the problem where a slightly stale
+    // catalog exists, but aircraft are modified - this causes an MD5 sum
+    // mismatch
+    packageRoot->refresh(true);
   }
-#endif // of ENABLE_PACKAGE_SYSTEM
+
+    _inited = true;
+}
+
+namespace {
+
+std::string _getDefaultCatalogId()
+{
+    return fgGetString("/sim/package-system/default-catalog/id", "org.flightgear.official" );
+}
+
+pkg::CatalogRef getDefaultCatalog()
+{
+    if (!globals->packageRoot())
+        return pkg::CatalogRef();
+
+    return globals->packageRoot()->getCatalogById(_getDefaultCatalogId());
+}
+
+} // of anonymous namespace
+
+bool FGHTTPClient::isDefaultCatalogInstalled() const
+{
+    return getDefaultCatalog().valid();
+}
+
+void FGHTTPClient::addDefaultCatalog()
+{
+    pkg::CatalogRef defaultCatalog = getDefaultCatalog();
+    if (!defaultCatalog.valid()) {
+      pkg::Catalog::createFromUrl(globals->packageRoot(), getDefaultCatalogUrl());
+    }
+}
+
+std::string FGHTTPClient::getDefaultCatalogId() const
+{
+    return _getDefaultCatalogId();
+}
+
+std::string FGHTTPClient::getDefaultCatalogUrl() const
+{
+    return fgGetString("/sim/package-system/default-catalog/url",
+                       "http://fgfs.goneabitbursar.com/pkg/" FLIGHTGEAR_VERSION "/catalog.xml");;
 }
 
 static naRef f_package_existingInstall( pkg::Package& pkg,
@@ -228,7 +272,6 @@ static naRef f_package_variants(pkg::Package& pack, naContext c)
 
 void FGHTTPClient::postinit()
 {
-#ifdef ENABLE_PACKAGE_SYSTEM
   NasalPackageRoot::init("PackageRoot")
   .member("path", &pkg::Root::path)
   .member("version", &pkg::Root::catalogVersion)
@@ -288,12 +331,19 @@ void FGHTTPClient::postinit()
     nasal::Hash nasalPkg = nasalGlobals.createHash("pkg"); // module
     nasalPkg.set("root", packageRoot);
   }
-#endif // of ENABLE_PACKAGE_SYSTEM
 }
 
 void FGHTTPClient::shutdown()
 {
-  _http.reset();
+    pkg::Root* packageRoot = globals->packageRoot();
+    if (packageRoot && _packageDelegate.get()) {
+        packageRoot->removeDelegate(_packageDelegate.get());
+    }
+
+    _packageDelegate.reset();
+    _http.reset();
+
+    _inited = false;
 }
 
 void FGHTTPClient::update(double)
