@@ -28,6 +28,8 @@
 #include <algorithm>
 #include <functional>
 
+#include <boost/lexical_cast.hpp>
+
 #include <osgViewer/Viewer>
 #include <osgDB/Registry>
 
@@ -126,14 +128,7 @@ FGTileMgr::FGTileMgr():
 
 FGTileMgr::~FGTileMgr()
 {
-    delete _listener;
-    
-    // remove all nodes we might have left behind
-    osg::Group* group = globals->get_scenery()->get_terrain_branch();
-    group->removeChildren(0, group->getNumChildren());
-    // clear OSG cache
-    osgDB::Registry::instance()->clearObjectCache();
-}
+   }
 
 
 // Initialize the Tile Manager subsystem
@@ -142,6 +137,21 @@ void FGTileMgr::init()
     reinit();
 }
 
+void FGTileMgr::shutdown()
+{
+    delete _listener;
+    _listener = NULL;
+
+    FGScenery* scenery = globals->get_scenery();
+    if (scenery && scenery->get_terrain_branch()) {
+        osg::Group* group = scenery->get_terrain_branch();
+        group->removeChildren(0, group->getNumChildren());
+    }
+    // clear OSG cache
+    osgDB::Registry::instance()->clearObjectCache();
+    state = Start; // need to init again
+}
+
 void FGTileMgr::reinit()
 {
     SG_LOG( SG_TERRAIN, SG_INFO, "Initializing Tile Manager subsystem." );
@@ -166,7 +176,8 @@ void FGTileMgr::reinit()
     
     if (!_disableNasalHooks->getBoolValue())
       _options->setModelData(new FGNasalModelDataProxy);
-  
+
+    _options->setPluginStringData("SimGear::ROUGH_LOD_RANGE", fgGetString("/sim/rendering/static-lod/rough", boost::lexical_cast<string>(SG_OBJECT_RANGE)));
   
     if (state != Start)
     {
@@ -219,6 +230,9 @@ bool FGTileMgr::sched_tile( const SGBucket& b, double priority, bool current_vie
     {
         // create a new entry
         t = new TileEntry( b );
+        SG_LOG( SG_TERRAIN, SG_INFO, "sched_tile: new tile entry for:" << b );
+
+
         // insert the tile into the cache, update will generate load request
         if ( tile_cache.insert_tile( t ) )
         {
@@ -469,7 +483,7 @@ void FGTileMgr::schedule_tiles_at(const SGGeod& location, double range_m)
             // We've moved to a new bucket, we need to schedule any
             // needed tiles for loading.
             SG_LOG( SG_TERRAIN, SG_INFO, "FGTileMgr: at " << location << ", scheduling needed for:" << current_bucket
-                   << ", visbility=" << range_m);
+                   << ", visibility=" << range_m);
             scheduled_visibility = range_m;
             schedule_needed(current_bucket, range_m);
         }
