@@ -39,8 +39,7 @@ typedef nasal::Ghost<simgear::HTTP::MemoryRequestRef> NasalMemoryRequest;
 
 FGHTTPClient& requireHTTPClient(naContext c)
 {
-  FGHTTPClient* http =
-    static_cast<FGHTTPClient*>(globals->get_subsystem("http"));
+  FGHTTPClient* http = globals->get_subsystem<FGHTTPClient>();
   if( !http )
     naRuntimeError(c, "Failed to get HTTP subsystem");
 
@@ -78,6 +77,15 @@ static naRef f_http_load(const nasal::CallContext& ctx)
   return ctx.to_nasal( requireHTTPClient(ctx.c).client()->load(url) );
 }
 
+static naRef f_request_abort(simgear::HTTP::Request&, const nasal::CallContext& ctx)
+{
+    // we need a request_ptr for cancel, not a reference. So extract
+    // the me object from the context directly.
+    simgear::HTTP::Request_ptr req = ctx.from_nasal<simgear::HTTP::Request_ptr>(ctx.me);
+    requireHTTPClient(ctx.c).client()->cancelRequest(req);
+    return naNil();
+}
+
 //------------------------------------------------------------------------------
 naRef initNasalHTTP(naRef globals, naContext c)
 {
@@ -94,7 +102,7 @@ naRef initNasalHTTP(naRef globals, naContext c)
     .member("status", &Request::responseCode)
     .member("reason", &Request::responseReason)
     .member("readyState", &Request::readyState)
-    .method("abort", static_cast<void (Request::*)()>(&Request::abort))
+    .method("abort", f_request_abort)
     .method("done", static_cast<HTTPCallback>(&Request::done))
     .method("fail", static_cast<HTTPCallback>(&Request::fail))
     .method("always", static_cast<HTTPCallback>(&Request::always));
