@@ -44,13 +44,14 @@
 #include <simgear/props/props.hxx>
 #include <simgear/structure/subsystem_mgr.hxx>
 #include <simgear/xml/easyxml.hxx>
+#include <simgear/timing/sg_time.hxx>
 
 #include <AIModel/AIFlightPlan.hxx>
 #include <AIModel/AIManager.hxx>
 #include <AIModel/AIAircraft.hxx>
 #include <Airports/airport.hxx>
-#include <Main/fg_init.hxx>   // That's pretty ugly, but I need fgFindAirportID
-
+#include <Main/globals.hxx>
+#include <Main/fg_props.hxx>
 
 #include "SchedFlight.hxx"
 #include "TrafficMgr.hxx"
@@ -403,10 +404,12 @@ void FGAISchedule::setHeading()
     courseToDest = SGGeodesy::courseDeg((*flights.begin())->getDepartureAirport()->geod(), (*flights.begin())->getArrivalAirport()->geod());
 }
 
+void FGAISchedule::assign(FGScheduledFlight *ref) { flights.push_back(ref); }
+
 bool FGAISchedule::scheduleFlights(time_t now)
 {
-  //string startingPort;
-  const string& userPort = fgGetString("/sim/presets/airport-id");
+    //string startingPort;
+    const string& userPort = fgGetString("/sim/presets/airport-id");
   SG_LOG(SG_AI, SG_BULK, "Scheduling Flights for : " << modelPath << " " <<  registration << " " << homePort);
   FGScheduledFlight *flight = NULL;
   SGTimeStamp start;
@@ -503,11 +506,59 @@ bool FGAISchedule::next()
    return true;
 }
 
+time_t FGAISchedule::getDepartureTime()
+{
+    if (flights.empty())
+        return 0;
+
+    return (*flights.begin())->getDepartureTime   ();
+}
+
+FGAirport *FGAISchedule::getDepartureAirport()
+{
+    if (flights.empty())
+        return 0;
+
+    return (*flights.begin())->getDepartureAirport();
+}
+
+FGAirport *FGAISchedule::getArrivalAirport()
+{
+    if (flights.empty())
+        return 0;
+
+    return (*flights.begin())->getArrivalAirport  ();
+}
+
+int FGAISchedule::getCruiseAlt()
+{
+    if (flights.empty())
+        return 0;
+
+    return (*flights.begin())->getCruiseAlt       ();
+}
+
+std::string FGAISchedule::getCallSign()
+{
+    if (flights.empty())
+        return std::string();
+
+    return (*flights.begin())->getCallSign ();
+}
+
+std::string FGAISchedule::getFlightRules()
+{
+    if (flights.empty())
+        return std::string();
+
+    return (*flights.begin())->getFlightRules ();
+}
+
 FGScheduledFlight* FGAISchedule::findAvailableFlight (const string &currentDestination,
                                                       const string &req,
                                                      time_t min, time_t max)
 {
-    time_t now = time(NULL) + fgGetLong("/sim/time/warp");
+    time_t now = globals->get_time_params()->get_cur_time();
 
     FGTrafficManager *tmgr = (FGTrafficManager *) globals->get_subsystem("traffic-manager");
     FGScheduledFlightVecIterator fltBegin, fltEnd;
@@ -602,7 +653,10 @@ double FGAISchedule::getSpeed()
   double dist = SGGeodesy::distanceNm(dep->geod(), arr->geod());
   double remainingTimeEnroute = (*i)->getArrivalTime() - (*i)->getDepartureTime();
 
-  double speed = dist / (remainingTimeEnroute/3600.0);
+  double speed = 0.0;
+  if (remainingTimeEnroute > 0.01) 
+    speed = dist / (remainingTimeEnroute/3600.0);
+
   SG_CLAMP_RANGE(speed, 300.0, 500.0);
   return speed;
 }
