@@ -97,12 +97,15 @@
 #include <Scenery/scenery.hxx>
 #include <Scenery/redout.hxx>
 #include <GUI/new_gui.hxx>
+#include <GUI/gui.h>
+
 #include <Instrumentation/HUD/HUD.hxx>
 #include <Environment/precipitation_mgr.hxx>
 #include <Environment/environment_mgr.hxx>
+#include <Environment/ephemeris.hxx>
 
 //#include <Main/main.hxx>
-#include "viewer.hxx"
+#include "view.hxx"
 #include "viewmgr.hxx"
 #include "splash.hxx"
 #include "renderer.hxx"
@@ -1446,7 +1449,10 @@ FGRenderer::setupView( void )
 
     setupRoot();
   
-// build the sky    
+// build the sky
+    Ephemeris* ephemerisSub = globals->get_subsystem<Ephemeris>();
+
+
     // The sun and moon diameters are scaled down numbers of the
     // actual diameters. This was needed to fit both the sun and the
     // moon within the distance to the far clip plane.
@@ -1457,7 +1463,7 @@ FGRenderer::setupView( void )
     opt->setPropertyNode(globals->get_props());
     _sky->build( 80000.0, 80000.0,
                   463.3, 361.8,
-                  *globals->get_ephem(),
+                  *ephemerisSub->data(),
                   fgGetNode("/environment", true),
                   opt.get());
     
@@ -1588,6 +1594,8 @@ FGRenderer::update( ) {
         sAlpha -= SGMiscd::max(0.0,delay_time/fade_time);
         FGScenerySwitchCallback::scenery_enabled = (sAlpha<1.0);
         _splash_alpha->setDoubleValue((sAlpha < 0) ? 0.0 : sAlpha);
+
+        syncPausePopupState();
     }
 
     FGLight *l = static_cast<FGLight*>(globals->get_subsystem("lighting"));
@@ -1610,7 +1618,7 @@ FGRenderer::update( ) {
     // initializations and are running the main loop, so this will
     // now work without seg faulting the system.
 
-    FGViewer *current__view = globals->get_current_view();
+    flightgear::View *current__view = globals->get_current_view();
     // Force update of center dependent values ...
     current__view->set_dirty();
   
@@ -1681,6 +1689,8 @@ FGRenderer::updateSky()
     } else {
         sun_horiz_eff = moon_horiz_eff = 1.0;
     }
+
+
     
     SGSkyState sstate;
     sstate.pos       = globals->get_current_view()->getViewPosition();
@@ -1699,10 +1709,11 @@ FGRenderer::updateSky()
     scolor.cloud_color = SGVec3f(l->cloud_color().data());
     scolor.sun_angle   = l->get_sun_angle();
     scolor.moon_angle  = l->get_moon_angle();
-    
+
+    Ephemeris* ephemerisSub = globals->get_subsystem<Ephemeris>();
     double delta_time_sec = _sim_delta_sec->getDoubleValue();
-    _sky->reposition( sstate, *globals->get_ephem(), delta_time_sec );
-    _sky->repaint( scolor, *globals->get_ephem() );
+    _sky->reposition( sstate, *ephemerisSub->data(), delta_time_sec );
+    _sky->repaint( scolor, *ephemerisSub->data() );
 }
                                     
 void
@@ -2061,7 +2072,7 @@ bool printVisibleSceneInfo(FGRenderer* renderer)
     VisibleSceneInfoVistor vsv;
     Viewport* vp = 0;
     if (!viewer->getCamera()->getViewport() && viewer->getNumSlaves() > 0) {
-        const View::Slave& slave = viewer->getSlave(0);
+        const osg::View::Slave& slave = viewer->getSlave(0);
         vp = slave._camera->getViewport();
     }
     vsv.doTraversal(viewer->getCamera(), viewer->getSceneData(), vp);
